/***************************************************************************
                          array3ditem.cpp  -  description
                             -------------------
    begin                : Tue Dec 25 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <float.h>
#include <KConfigGroup>
#include <KMessageBox>
#include "fitpack.h"
#include "splineitem.h"
#include "array3ditem.h"

static int sortCol;

/**
 * Compares table rows for qsort.
 * @param e1 pointer to row 1.
 * @param e2 pointer to row 2.
 * @return -1 if x1 < x2, 0 if x1 = x2, 1 if x1 > x2.
 */
int cmpasc(const void *e1, const void *e2)
{
  double a1 = (*((double**) e1))[sortCol];
  double a2 = (*((double**) e2))[sortCol];
  if (a1 < a2)
    return -1;
  if (a1 > a2)
    return 1;
  return 0;
}

Array3DItem::Array3DItem(double x0, double y0, double width, double height,
                         unsigned colorFrame, unsigned colorGrid,
                         unsigned colorData, bool logX, bool logY, bool logZ,
                         GridMode gridMode, Mode3D mode3D, double xMax,
                         double yMax, double zMax, const QString& textX,
                         const QString& textY, const QString& textZ,
                         const QString& header, double relSize,
                         int colX, int colY, int colZ, int colErr,
                         bool errorBars,
                         ArrayItem::DecimalCharacter decimalChar, int numDigX,
                         int numDigY, int numDigZ, int minTicX, int minTicY,
                         int minTicZ, int expX, int expY, int expZ,
                         double xMin, double yMin, double zMin, double xTic,
                         double yTic, double zTic, double deltaX,
                         double deltaZ, double phi, double theta, double boxX,
                         double boxY, double boxZ, double xRef, double yRef,
                         double fx, double fy, double fz, int rowStart, int n,
                         double smooth, const QUrl& url, bool showFrame,
                         bool internal, bool collapsed, bool visible) :
 ThreeDItem(x0, y0, width, height, colorFrame, colorGrid, colorData, logX,
            logY, logZ, gridMode, mode3D, xMax, yMax, zMax, textX, textY,
            textZ, header, relSize, numDigX, numDigY, numDigZ, minTicX,
            minTicY, minTicZ, expX, expY, expZ, xMin, yMin, zMin, xTic, yTic,
            zTic, deltaX, deltaZ, phi, theta, boxX, boxY, boxZ, xRef, yRef, fx,
            fy, fz, showFrame, collapsed, visible), m_internal(internal),
 m_errorBars(errorBars), errorBarso(0), m_colX(colX), m_colY(colY),
 m_colZ(colZ), m_colErr(colErr), m_rowStart(rowStart), m_count(n),
 m_numRows(0), m_numCols(0), m_decimalCharacter(decimalChar), colXo(0),
 colYo(0), colZo(0), colErro(0), rowStarto(0), counto(0), m_smf(smooth),
 smfo(0), m_data(0), m_url(url)
{
  if (m_internal) {
    m_count = m_numRows = n;
    m_numCols = qMin(3, qMax(qMax(qMax(colX, colY), colZ), colErr) + 1);
    m_decimalCharacter = ArrayItem::Point;
    m_data = new double*[m_numCols];
    int dim = m_numCols * m_numRows;
    m_data[0] = new double[dim];
    memset(m_data[0], 0, dim * sizeof(double));
    for (int j = 1; j < m_numCols; ++j)
      m_data[j] = m_data[j - 1] + m_numRows;
  } else
    if (!m_url.isEmpty()) {
      m_numRows = ArrayItem::readFile(m_url, m_numCols, &m_data,
                                      m_decimalCharacter);
      if (!m_numRows) {
        KMessageBox::error(0, m_url.url());
          return;
      }
      m_rowStart = qMin(rowStart, m_numRows - 1);
      m_count = qMin(n, m_numRows - m_rowStart);
    }
}

Array3DItem::Array3DItem(const Array3DItem& a) :
 ThreeDItem(a), m_internal(a.m_internal), m_errorBars(a.m_errorBars),
 errorBarso(a.errorBarso), m_colX(a.m_colX), m_colY(a.m_colY),
 m_colZ(a.m_colZ), m_colErr(a.m_colErr), m_rowStart(a.m_rowStart),
 m_count(a.m_count), m_numRows(a.m_numRows), m_numCols(a.m_numCols),
 m_decimalCharacter(a.m_decimalCharacter), colXo(a.colXo), colYo(a.colYo),
 colZo(a.colZo), colErro(a.colErro), rowStarto(a.rowStarto), counto(a.counto),
 m_smf(a.m_smf), smfo(a.smfo),
 m_data(ArrayItem::copyArray(a.m_data, a.m_numCols, a.m_numRows)),
 m_url(a.m_url)
{
}

Array3DItem::Array3DItem(const KConfigGroup* plo, const QUrl& urlPlo,
                         double x0, double y0, double width, double height,
                         unsigned colorFrame, unsigned colorGrid,
                         unsigned colorData, GridMode gridMode,
                         int colX, int colY, int colErr, bool errorBars) :
 ThreeDItem(plo, x0, y0, width, height, colorFrame, colorGrid, colorData,
            gridMode), errorBarso(0), m_rowStart(0), colXo(0), colYo(0),
 colZo(0), colErro(0), rowStarto(0), counto(0), smfo(0), m_data(0)
{
  m_internal = plo->readEntry("internal", false);
  if (m_internal) {
    m_count = m_numRows = plo->readEntry("n", 0);
    m_numCols = plo->readEntry("r0").split(' ').count();
    m_decimalCharacter = ArrayItem::Point;
    m_data = new double*[m_numCols];
    m_data[0] = new double[m_numCols * m_numRows];
    for (int j = 1; j < m_numCols; ++j)
      m_data[j] = m_data[j - 1] + m_numRows;
    for (int i = 0; i < m_numRows; ++i) {
      QStringList list = plo->readEntry(QString("r%1").arg(i)).split(' ');
      int cnt = list.count();
      for (int j = 0; j < m_numCols; ++j)
        m_data[j][i] = (j < cnt) ? list[j].toDouble() : 0;
    }
  } else {
    m_url = absoluteUrl(plo->readEntry("path", ""), urlPlo);
    m_decimalCharacter = static_cast<ArrayItem::DecimalCharacter>(
      plo->readEntry("idec", 0));
    m_numRows = ArrayItem::readFile(m_url, m_numCols, &m_data,
                                    m_decimalCharacter);
    if (!m_numRows) {
      KMessageBox::error(0, m_url.url());
      return;
    }
    m_rowStart = qMin(plo->readEntry("istart", 0), m_numRows - 1);
    int nmax =  m_numRows - m_rowStart;
    m_count = qMin(plo->readEntry("n", nmax), nmax);
  }
  int icolmax = m_numCols - 1;
  m_colX = plo->readEntry("ix", qMin(colX, icolmax));
  m_colY = plo->readEntry("iy", qMin(colY, icolmax));
  m_colZ = plo->readEntry("iz", qMin(1, icolmax));
  m_colErr = plo->readEntry("ie", qMin(colErr, icolmax));
  m_errorBars = plo->readEntry("errbars", errorBars);
  m_smf = plo->readEntry("smooth", 1.0);
}

Array3DItem::~Array3DItem()
{
  ArrayItem::freeArray(&m_data);
}

void Array3DItem::calcTable() const
{
  if (!m_data)
    return;
  bool newv = !m_yValues;
  if (m_deltaX) {
    nx = int(fabs((m_xMax - m_xMin) / m_deltaX)) + 1;
    xStep = m_logX ? (log10(m_xMax / m_xMin) / (nx - 1)) : m_deltaX;
  } else {
    nx = 41;
    xStep = 0.025 * (m_logX ? log10(m_xMax / m_xMin) : (m_xMax - m_xMin));
  }
  if (m_deltaZ) {
    nz = int(fabs((m_zMax - m_zMin) / m_deltaZ)) + 1;
    zStep = m_logZ ? (log10(m_zMax / m_zMin) / (nz - 1)) : m_deltaZ;
  } else {
    nz = 41;
    zStep = 0.025 * (m_logZ ? log10(m_zMax / m_zMin) : (m_zMax - m_zMin));
  }
  if (newv || (m_xMin != xMino) || (m_xMax != xMaxo) ||
      (m_deltaX != deltaXo) || (m_logX != logXo) || (m_zMin != zMino) ||
      (m_zMax != zMaxo) || (m_deltaZ != deltaZo) || (m_logZ != logZo) ||
      (m_colX != colXo) || (m_colY != colYo) || (m_colZ != colZo) ||
      (m_rowStart != rowStarto) || (m_count != counto) ||
      (m_colErr != colErro) || (m_errorBars != errorBarso) ||
      (m_smf != smfo)) {
    ArrayItem::freeArray(&m_yValues);
    double* y1 = new double[nx * nz];
    m_yValues = new double*[nx];
    for (int i = 0; i < nx; ++i)
      m_yValues[i] = &y1[i * nz];
    int nkx, nkz, ier;
    double xmin, xmax, zmin, zmax, fp;
    ScaledItem::limits(xmin, xmax, &m_data[m_colX][m_rowStart], m_count);
    ScaledItem::limits(zmin, zmax, &m_data[m_colZ][m_rowStart], m_count);
    xmin = qMin(m_xMin, xmin);
    xmax = qMax(m_xMax, xmax);
    if (m_logX) {
      xmin = log10(xmin);
      xmax = log10(xmax);
    }
    zmin = qMin(m_zMin, zmin);
    zmax = qMax(m_zMax, zmax);
    if (m_logZ) {
      zmin = log10(zmin);
      zmax = log10(zmax);
    }
    QVector<double> xi(nx);
    for (int i = 0; i < nx; ++i)
      xi[i] = (m_logX ? log10(m_xMin) : m_xMin) + i * xStep;
    QVector<double> zi(nz);
    for (int i = 0; i < nz; ++i)
      zi[i] = (m_logZ ? log10(m_zMin) : m_zMin) + i * zStep;
    if (!m_smf) {
      double* a1 = new double[3 * m_count];
      double** a2 = new double*[m_count];
      for (int i = 0; i < m_count; ++i) {
        a2[i] = &a1[3 * i];
        a2[i][0] = m_logX ? log10(m_data[m_colX][m_rowStart + i]) :
                            m_data[m_colX][m_rowStart + i];
        a2[i][1] = m_logZ ? log10(m_data[m_colZ][m_rowStart + i]) :
                            m_data[m_colZ][m_rowStart + i];
        a2[i][2] = m_data[m_colY][m_rowStart + i];
      }
      sortCol = 1;
      qsort(a2, m_count, sizeof(double*), cmpasc);
      int ib = 0;
      nkz = 0;
      QVector<double> z;
      QVector<double*> c2;
      while (ib < m_count) {
        int j = ib + 1;
        while ((j < m_count) && (a2[j][1] == a2[j - 1][1]))
          j++;
        nkx = j - ib;
        double* b1 = new double[2 * nkx];
        double** b2 = new double*[nkx];
        for (int i = 0; i < nkx; ++i) {
          b2[i] = &b1[2 * i];
          b2[i][0] = a2[ib + i][0];
          b2[i][1] = a2[ib + i][2];
        }
        sortCol = 0;
        qsort(b2, nkx, sizeof(double*), cmpasc);
        QVector<double> x(nkx);
        QVector<double> y(nkx);
        for (int i = 0; i < nkx; ++i) {
          x[i] = b2[i][0];
          y[i] = b2[i][1];
        }
        SplineItem spl(qMin(3, nkx - 1));
        ier = spl.interpolation(x.data(), y.data(), nkx, xmin, xmax);
        if (ier > 3)
          break;
        z.resize(nkz + 1);
        z[nkz] = a2[ib][1];
        c2.resize(nkz + 1);
        c2[nkz] = new double[nx];
        spl.calcTable(xi.data(), c2[nkz++], nx);
        delete [] b1;
        delete [] b2;
        ib += nkx;
      }
      if (ier < 4) {
        QVector<double> y(nkz);
        for (int j = 0; j < nx; ++j) {
          for (int i = 0; i < nkz; ++i)
            y[i] = c2[i][j];
          SplineItem spl(qMin(3, nkz - 1));
          ier = spl.interpolation(z.data(), y.data(), nkz, zmin, zmax);
          if (ier < 4)
            spl.calcTable(zi.data(), m_yValues[j], nz);
        }
      }
      delete [] a1;
      delete [] a2;
    } else {
      int k = 3;
      int k1 = k + 1;
      int nest = qMin(m_count, 121) + k1;
      int nk = nest - k1;
      QVector<double> x(m_count);
      QVector<double> z(m_count);
      QVector<double> wt(m_count);
      bool errcol = m_errorBars && (m_colErr < m_numCols);
      for (int i = 0; i < m_count; ++i) {
        x[i] = m_data[m_colX][m_rowStart + i];
        if (m_logX)
           x[i] = log10(x[i]);
        z[i] = m_data[m_colZ][m_rowStart + i];
        if (m_logZ)
           z[i] = log10(z[i]);
        wt[i] = errcol ? (1 / m_data[m_colErr][m_rowStart + i]) : 1;
      }
      QVector<double> tx(nest);
      QVector<double> tz(nest);
      QVector<double> c(nk * nk);
      FitPack::surfit(m_count, x.data(), z.data(),
                      &m_data[m_colY][m_rowStart], wt.data(), xmin, xmax, zmin,
                      zmax, k, k, m_smf * m_smf * m_count, nest, nest, nest,
                      DBL_EPSILON, &nkx, tx.data(), &nkz, tz.data(), c.data(),
                      &fp, &ier);
      if (ier < 6)
        FitPack::bispev(tx.data(), nkx, tz.data(), nkz, c.data(), k, k,
                        xi.data(), nx, zi.data(), nz, m_yValues[0], &ier);
    }
  }
  xMino = m_xMin;
  xMaxo = m_xMax;
  deltaXo = m_deltaX;
  logXo = m_logX;
  zMino = m_zMin;
  zMaxo = m_zMax;
  deltaZo = m_deltaZ;
  logZo = m_logZ;
  colXo = m_colX;
  colYo = m_colY;
  colZo = m_colZ;
  colErro = m_colErr;
  rowStarto = m_rowStart;
  counto = m_count;
  errorBarso = m_errorBars;
  smfo = m_smf;
}

void Array3DItem::paint(QPainter* painter,
                        const QStyleOptionGraphicsItem* option,
                        QWidget* widget)
{
  calcTable();
  ThreeDItem::paint(painter, option, widget);
}

void Array3DItem::update()
{
  calcTable();
  ThreeDItem::update();
}

void Array3DItem::writePlo(KConfigGroup* plo, const QUrl& urlPlo,
                           bool absolute) const
{
  ThreeDItem::writePlo(plo);
  plo->writeEntry("Type", "ARRAY3DITEM");
  plo->writeEntry("internal", m_internal);
  plo->writeEntry("ix", m_colX);
  plo->writeEntry("iy", m_colY);
  plo->writeEntry("iz", m_colZ);
  plo->writeEntry("ie", m_colErr);
  plo->writeEntry("errbars", m_errorBars ? 1 : 0);
  plo->writeEntry("smooth", m_smf);
  plo->writeEntry("n", m_count);
  if (m_internal) {
    plo->writeEntry("istart", 0);
    for (int i = 0; i < m_count; i++) {
      QStringList list;
      for (int j = 0; j < m_numCols; j++)
        list.append(QString::number(m_data[j][i + m_rowStart]));
      plo->writeEntry(QString("r%1").arg(i), list.join(" "));
    }
  } else {
    plo->writeEntry("idec", int(m_decimalCharacter));
    plo->writeEntry("istart", m_rowStart);
    plo->writeEntry("path", path(urlPlo, m_url, absolute));
  }
}
