/***************************************************************************
                          rectitem.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QBrush>
#include <KConfigGroup>
#include "kplscene.h"
#include "rectitem.h"

RectItem::RectItem(double x0, double y0, unsigned color, QGraphicsItem* parent,
                   double width, double height, bool visible, FillStyle fill,
                   Symbol symbol, double relSize, double dir) :
 DecoItem(color, x0, y0, relSize), QGraphicsRectItem(parent),
 m_fillStyle(fill), m_width(width), m_height(height), m_dir(dir),
 m_symbol(symbol)
{
  if (!visible)
    hide();
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

RectItem::RectItem(const RectItem& a) :
 DecoItem(a), QGraphicsRectItem(), m_fillStyle(a.m_fillStyle), m_width(a.m_width),
 m_height(a.m_height), m_dir(a.m_dir), m_symbol(a.m_symbol)
{
  setZValue(a.zValue());
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
  setSelected(a.isSelected());
  setVisible(a.isVisible());
}

RectItem::RectItem(const KConfigGroup* plo, unsigned color,
                   QGraphicsItem* parent) :
 DecoItem(plo, color), QGraphicsRectItem(parent)
{
  if (!plo->readEntry("active", true))
    hide();
  m_fillStyle = FillStyle(plo->readEntry("fillstyle", 0));
  if (!m_fillStyle)
    if (plo->readEntry("filled", false))
      m_fillStyle = SolidPattern;
  m_width = plo->readEntry("width", 3.0);
  m_height = plo->readEntry("height", 2.0);
  m_dir = plo->readEntry("dir", 0.0);
  m_symbol = Symbol(plo->readEntry("symb", int(SolidLine)));
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

RectItem::~RectItem()
{
}

QPixmap RectItem::pixmap() const
{
  return KplItem::pixmap(QPen(m_color),
                         m_fillStyle ? FilledSquare : OpenSquare);
}

void RectItem::setPenWidth()
{
  if (!m_fillStyle) {
    QPen p = pen();
    double w = penWidth(this);
    if (p.widthF() != w) {
      prepareGeometryChange();
      setPen(changePenWidth(p, w));
    }
  }
}

void RectItem::setProperties(const QTransform& m)
{
  m_matrix = m;
  setProperties();
}

void RectItem::update()
{
  prepareGeometryChange();
  setProperties();
  QGraphicsItem::update();
}

void RectItem::writePlo(KConfigGroup* plo) const
{
  plo->writeEntry("Type", "RECTITEM");
  DecoItem::writePlo(plo, isVisible());
  plo->writeEntry("fillstyle", int(m_fillStyle));
  plo->writeEntry("width", m_width);
  plo->writeEntry("height", m_height);
  if (m_dir)
    plo->writeEntry("dir", m_dir);
  plo->writeEntry("symb", int(m_symbol));
}

void RectItem::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
  QGraphicsRectItem::mouseReleaseEvent(event);
  ((KplScene*)scene())->updatePos(this);
}

void RectItem::setProperties()
{
  double h = qAbs(m_matrix.m22() * m_height);
  setRect(0, -h, m_matrix.m11() * m_width, h);
  setPos(m_matrix.map(QPointF(m_x0, m_y0)));
  setRotation(-m_dir);
  if (m_fillStyle) {
    setBrush(createBrush(m_color, m_fillStyle));
    setPen(Qt::NoPen);
  } else
    setPen(createPen(this, m_color, DataPen, m_symbol));
}
