/***************************************************************************
                          threeditem.cpp  -  description
                             -------------------
    begin                : Tue Dec 25 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <float.h>
#include <QtMath>
#include <QPainter>
#include <QTextStream>
#include <KConfigGroup>
#include "kplscene.h"
#include "psgraph.h"
#include "threeditem.h"

static const int nRes3D = 2000;

ThreeDItem::ThreeDItem(double x0, double y0, double width, double height,
                       unsigned colorFrame, unsigned colorGrid,
                       unsigned colorData, bool logX, bool logY, bool logZ,
                       GridMode gridMode, Mode3D mode3D, double xMax,
                       double yMax, double zMax, const QString& textX,
                       const QString& textY, const QString& textZ,
                       const QString& header, double relSize, int numDigX,
                       int numDigY, int numDigZ, int minTicX, int minTicY,
                       int minTicZ, int expX, int expY, int expZ, double xMin,
                       double yMin, double zMin, double xTic, double yTic,
                       double zTic, double deltaX, double deltaZ, double phi,
                       double theta, double boxX, double boxY, double boxZ,
                       double xRef, double yRef, double fx, double fy,
                       double fz, bool showFrame, bool collapsed,
                       bool visible) :
 FrameItem(x0, y0, width, height, colorFrame, colorGrid, logX, logY, gridMode,
           xMax, yMax, textX, textY, header, relSize, numDigX, numDigY,
           minTicX, minTicY, expX, expY, xMin, yMin, xTic, yTic, collapsed,
           visible), m_showFrame(showFrame), m_logZ(logZ), logXo(false),
 logZo(false), m_numDigZ(numDigZ), m_minTicZ(minTicZ), m_expZ(expZ),
 m_mode3D(mode3D), m_colorData(colorData), m_zMin(zMin), m_zMax(zMax),
 m_deltaX(deltaX), m_deltaZ(deltaZ), m_zTic(zTic), m_phi(phi), m_theta(theta),
 m_boxX(boxX), m_boxY(boxY), m_boxZ(boxZ), m_xRef(xRef), m_yRef(yRef),
 m_fX(fx), m_fY(fy), m_fZ(fz), xMino(0), xMaxo(0), deltaXo(0), zMino(0),
 zMaxo(0), deltaZo(0), phio(1e300), thetao(1e300), m_yValues(0), m_textZ(textZ)
{
  yMin3D.resize(nRes3D);
  yMax3D.resize(nRes3D);
}

ThreeDItem::ThreeDItem(const ThreeDItem& a) :
 FrameItem(a), m_showFrame(a.m_showFrame), m_logZ(a.m_logZ), logXo(a.logXo),
 logZo(a.logZo), m_numDigZ(a.m_numDigZ), m_minTicZ(a.m_minTicZ),
 m_expZ(a.m_expZ), m_mode3D(a.m_mode3D), nx(a.nx), nz(a.nz),
 m_colorData(a.m_colorData), m_zMin(a.m_zMin), m_zMax(a.m_zMax),
 m_deltaX(a.m_deltaX), m_deltaZ(a.m_deltaZ), m_zTic(a.m_zTic), m_phi(a.m_phi),
 m_theta(a.m_theta), m_boxX(a.m_boxX), m_boxY(a.m_boxY), m_boxZ(a.m_boxZ),
 m_xRef(a.m_xRef), m_yRef(a.m_yRef), m_fX(a.m_fX), m_fY(a.m_fY), m_fZ(a.m_fZ),
 zminl(a.zminl), xStep(a.xStep), zStep(a.zStep), xMino(a.xMino),
 xMaxo(a.xMaxo), deltaXo(a.deltaXo), zMino(a.zMino), zMaxo(a.zMaxo),
 deltaZo(a.deltaZo), scalx(a.scalx), scaly(a.scaly), scalz(a.scalz),
 phio(a.phio), thetao(a.thetao), m_yValues(0), m3D(a.m3D), m_textZ(a.m_textZ)
{
  yMin3D.resize(nRes3D);
  yMax3D.resize(nRes3D);
  m_yValues = ArrayItem::copyArray(a.m_yValues, a.nx, a.nz);
}

ThreeDItem::ThreeDItem(const KConfigGroup* plo, double x0, double y0,
                       double width, double height, unsigned colorFrame,
                       unsigned colorGrid, unsigned colorData,
                       GridMode gridMode) :
 FrameItem(plo, x0, y0, width, height, colorFrame, colorGrid, gridMode),
 logXo(false), logZo(false), xMino(0), xMaxo(0), deltaXo(0), zMino(0),
 zMaxo(0), deltaZo(0), phio(1e300), thetao(1e300), m_yValues(0)
{
  yMin3D.resize(nRes3D);
  yMax3D.resize(nRes3D);
  m_showFrame = plo->readEntry("frame", true);
  m_logZ = plo->readEntry("logz", false);
  m_numDigZ = plo->readEntry("ndigz", -1);
  m_minTicZ = plo->readEntry("mticz", 5);
  m_expZ = plo->readEntry("iez", 0);
  m_mode3D = Mode3D(plo->readEntry("mode3d", int(Grid)));
  QString s = plo->readEntry("coldata", "");
  m_colorData = s.isEmpty() ? colorData : rgb(s);
  m_zMin = plo->readEntry("zmi", 0.0);
  m_zMax = plo->readEntry("zma", m_zMin + 1);
  m_deltaX = plo->readEntry("dx", 0.0);
  m_deltaZ = plo->readEntry("dz", 0.0);
  m_zTic = plo->readEntry("ztic", 5.0);
  m_phi = plo->readEntry("phi", 20.0);
  m_theta = plo->readEntry("theta", -20.0);
  m_boxX = plo->readEntry("boxx", 7.5);
  m_boxY = plo->readEntry("boxy", 7.5);
  m_boxZ = plo->readEntry("boxz", 7.5);
  m_xRef = plo->readEntry("xref", 7.5);
  m_yRef = plo->readEntry("yref", 4.0);
  m_fX = plo->readEntry("fx", 1.0);
  m_fY = plo->readEntry("fy", 1.0);
  m_fZ = plo->readEntry("fz", 1.0);
  m_textZ = plo->readEntry("sz", "");
}

ThreeDItem::~ThreeDItem()
{
}

QRectF ThreeDItem::boundingRect() const
{
  paintCond(0);
  if (bRect.isNull())
    bRect = QGraphicsRectItem::boundingRect();
  else
    bRect |= QGraphicsRectItem::boundingRect();
  return bRect;
}

void ThreeDItem::exportTable(QTextStream& stream, char separator, char format,
                             int precision) const
{
  for (int ix = 0; ix < nx; ix++)
    for (int iz = 0; iz < nz; iz++)
      stream << QString::number(m_xMin + ix * xStep, format, precision)
             << separator
             << QString::number(m_zMin + iz * zStep, format, precision)
             << separator
             << QString::number(m_yValues[ix][iz], format, precision) << "\n";
}

void ThreeDItem::paint(QPainter* painter,
                       const QStyleOptionGraphicsItem* option, QWidget* widget)
{
  paintCond(painter);
  if (m_showFrame && m_yValues)
    QGraphicsRectItem::paint(painter, option, widget);
}

void ThreeDItem::paintCond(QPainter* painter, PSGraph* ps) const
{
  if (!m_yValues)
    return;
  if (painter) {
    QPen p = pen();
    p.setColor(m_colorData);
    painter->setPen(changePenWidth(p, p.widthF() * penWidthFactor(GridPen) /
                    penWidthFactor(FramePen)));
  } else
    if (!ps)
      bRect = QRectF();
  if (!ps)
    drawText(painter, m_header, 0.5 * m_width,
             m_height + 0.0295 * m_relSize * m_width, TextItem::AlignCentered,
             ps);
  scalx = m_boxX / (m_logX ? log10(m_xMax / m_xMin) :
                             (m_fX * (m_xMax - m_xMin)));
  scaly = m_boxY / (m_logY ? log10(m_yMax / m_yMin) :
                             (m_fY * (m_yMax - m_yMin)));
  scalz = m_boxZ / (m_logZ ? log10(m_zMax / m_zMin) :
                             (m_fZ * (m_zMax - m_zMin)));
  scal3D = (nRes3D - 1) / m_width;
  if ((m_phi != phio) || (m_theta != thetao)) {
    phio = m_phi;
    thetao = m_theta;
    double phi = qDegreesToRadians(m_phi);
    double theta = qDegreesToRadians(m_theta);
    double m13 = -sin(phi);
    double m11 = cos(phi);
    double s = sin(theta);
    m3D.setMatrix(m11, 0, m13, -s * m13, cos(theta), s * m11, 0, 0, 0);
  }
  if (ps)
    ps->setRelSize(0.707 * m_relSize);
  plotArray3D(painter, ps);
  if (painter)
    painter->setPen(pen());
  raster3d(painter, ps);
}

void ThreeDItem::setProperties(const QTransform& m)
{
  m_matrix = m;
  setProperties();
}

void ThreeDItem::update()
{
  setProperties();
  QGraphicsRectItem::update();
}

void ThreeDItem::writePlo(KConfigGroup* plo) const
{
  FrameItem::writePlo(plo);
  plo->writeEntry("frame", m_showFrame);
  if (m_logZ)
    plo->writeEntry("logz", m_logZ);
  plo->writeEntry("ndigz", m_numDigZ);
  plo->writeEntry("mticz", m_minTicZ);
  plo->writeEntry("iez", m_expZ);
  plo->writeEntry("mode3d", int(m_mode3D));
  plo->writeEntry("coldata", QString::number(rgbQt1(m_colorData), 16));
  plo->writeEntry("zmi", m_zMin);
  plo->writeEntry("zma", m_zMax);
  plo->writeEntry("dx", m_deltaX);
  plo->writeEntry("dz", m_deltaZ);
  plo->writeEntry("ztic", m_zTic);
  plo->writeEntry("phi", m_phi);
  plo->writeEntry("theta", m_theta);
  plo->writeEntry("boxx", m_boxX);
  plo->writeEntry("boxy", m_boxY);
  plo->writeEntry("boxz", m_boxZ);
  plo->writeEntry("xref", m_xRef);
  plo->writeEntry("yref", m_yRef);
  plo->writeEntry("fx", m_fX);
  plo->writeEntry("fy", m_fY);
  plo->writeEntry("fz", m_fZ);
  plo->writeEntry("sz", m_textZ);
}

void ThreeDItem::axis3D(QPainter* painter, double rmi, double rma, double tic,
                        int mtic, double f, double scal, double xa, double za,
                        double da2, int ndig, Axis axis, bool log,
                        PSGraph* ps) const
{
  TextItem::Alignment io = (axis == Y) ? TextItem::AlignRight :
                                         TextItem::AlignCentered;
  int kl = (axis == Y) ? 1 : 2;
  double rl2;
  double rl = 0;
  double mant = 0;
  double d2 = 0.0125 * m_relSize * m_width;
  double xArr[2], yArr[2];
  if (m_gridMode == AxesWithLabels) {
    switch (axis) {
      case X:
        if (fabs(m3D.m21()) > 0.8 * fabs(m3D.m11()))
          io = (((m3D.m23() * m3D.m22() * zStep < 0) ? -1 : 1) *
                m3D.m13() >= 0) ? TextItem::AlignRight : TextItem::AlignLeft;
        break;
      case Z:
        if (fabs(m3D.m23()) > 0.8 * fabs(m3D.m13()))
          io = (((m3D.m21() * m3D.m22() * xStep < 0) ? -1 : 1) *
                m3D.m11() >= 0) ? TextItem::AlignRight : TextItem::AlignLeft;
      case Y:
        break;
    }
  }
  if (log) {
    if (tic < 2.5)
      tic = 2;
    else
      if (tic < 5.5)
        tic = 3;
      else
        if (tic < 99)
          tic = 10;
  }
  bool e = log && (tic > 9.9) && ((rma > 0.999999e3) || (rmi < 1.000001e-3));
  for (double r = rmi; r < (rma + 0.00001 * (rma - rmi));) {
    if (ps)
      ps->setRelSize(0.707 * m_relSize);
    if (log) {
      rl = log10(r);
      rl2 = rl;
      mant = incLog(rl2, tic);
    }
    if (mtic > 1) {
      double d;
      double rm = 0;
      if (log)
        rm = rl;
      d = log ? pow(10, rm - mant) : (tic / (mtic * f));
      for (int i = 1;; ++i) {
        if (log) {
          rm = log10(pow(10, rm) + d);
          if ((rm >= rl2) || (rm >= log10(rma)))
            break;
        } else {
          rm = r + i * d;
          if (rm >= rma)
            break;
        }
        double r2 = rm * scal;
        switch (axis) {
          case X:
            line3D(painter, r2, 0, za, r2, 0, za + 0.5 * da2, ps);
            break;
          case Y:
            xArr[0] = xFrom3d(xa, za);
            xArr[1] = xArr[0] + 0.5 * da2;
            yArr[0] = yArr[1] = yFrom3d(xa, f * (r2 - scal * (log ?
                                                              log10(rmi) :
                                                              rmi)), za);
            plotArray(painter, xArr, yArr, 2, ps);
            break;
          case Z:
            line3D(painter, xa, 0, r2, xa + 0.5 * da2, 0, r2, ps);
        }
      }
    }
    double r2 = (log ? rl : r) * scal;
    if (r > rmi)
      switch (axis) {
        case X:
          if (r < rma)
            line3D(painter, r2, 0, za, r2, 0, za + da2, ps);
          break;
        case Y: {
            double xArr[2], yArr[2];
            xArr[0] = xFrom3d(xa, za);
            xArr[1] = xArr[0] + da2;
            yArr[0] = yArr[1] = yFrom3d(xa, f * (r2 - scal * (log ?
                                                              log10(rmi) :
                                                              rmi)), za);
            plotArray(painter, xArr, yArr, 2, ps);
          }
          break;
        case Z:
          if (r < rma)
            line3D(painter, xa, 0, r2, xa + da2, 0, r2, ps);
      }
    if (m_gridMode == AxesWithLabels) {
      if (ps)
        ps->setRelSize(m_relSize);
      QString label;
      double r2 = qMin(r, rma);
      if (log)
        ndig = log10(r2) < 0 ? (1 - int(log10(r2) + 0.01)) : -1;
      int l = e ? (label.sprintf("10#nu%d", qRound(log10(r2))).length() - 3) :
                   number(f * r2, ndig, label);
      if (io != 2)
        kl = qMax(kl, l);
      r2 = f * scal * (log ? rl : r);
      switch (axis) {
        case X: {
            double z = m_fZ * (za - 2.5 * da2);
            drawText(painter, label, xFrom3d(r2, z),
                     yFrom3d(r2, 0, z) - d2, io, ps);
          }
          break;
        case Y:
          drawText(painter, label, xFrom3d(xa, za) - d2,
                   yFrom3d(xa, r2 - f * scal * (log ? log10(rmi) : rmi), za) -
                   d2, io, ps);
          break;
        case Z:
          double x = m_fX * (xa - 2.5 * da2);
          drawText(painter, label, xFrom3d(x, r2), yFrom3d(x, 0, r2) - d2,
                   io, ps);
      }
    }
    r = log ? pow(10, rl2) : (r + tic / f);
  }
  if (m_gridMode == AxesWithLabels) {
    if (ps)
      ps->setRelSize(m_relSize);
    double phi = M_PI_2;
    switch (axis) {
      case X:
        if (m3D.m11())
          phi = atan(m3D.m21() / m3D.m11()) + M_PI + M_PI;
        break;
      case Z:
        if (m3D.m13())
          phi = atan(m3D.m23() / m3D.m13()) + M_PI + M_PI;
      case Y:
        break;
    }
    while (phi > 1.57097)
      phi -= M_PI;
    double r;
    if (axis == Y)
      r = log ? log10(rma / rmi) : (rma - rmi);
    else
      r = log ? log10(rmi * rma) : (rmi + rma);
    r *= 0.5 * f * scal;
    double d = 2 * (kl + 2) * d2 / m_relSize;
    double r2;
    double phi2 = qRadiansToDegrees(phi);
    switch (axis) {
      case X:
        r2 = m_fZ * za;
        if (m3D.m22() >= 0)
          d = -d;
        drawText(painter, m_expX ?
                 ("10#nu" + QString::number(m_expX) + "#nn " + m_textX) :
                 m_textX, xFrom3d(r, r2) - d * sin(phi),
                 yFrom3d(r, 0, r2) + d * cos(phi), TextItem::AlignCentered, ps,
                 phi2);
        break;
      case Y:
        drawText(painter, m_expY ?
                 ("10#nu" + QString::number(m_expY) + "#nn " + m_textY) :
                 m_textY, xFrom3d(xa, za) - 1.5 * (kl + 2) * d2,
                 yFrom3d(xa, r, za), TextItem::AlignCentered, ps, phi2);
        break;
      case Z:
        r2 = m_fX * xa;
        if (m3D.m22() <= 0)
          d = -d;
        drawText(painter, m_expZ ?
                 ("10#nu" + QString::number(m_expZ) + "#nn " + m_textZ) :
                 m_textZ, xFrom3d(r2, r) + d * sin(phi),
                 yFrom3d(r2, 0, r) - d * cos(phi), TextItem::AlignCentered, ps,
                 phi2);
    }
  }
}

void ThreeDItem::drawText(QPainter* painter, const QString& s, double x,
                          double y, TextItem::Alignment alignment, PSGraph* ps,
                          double dir) const
{
  if (ps) {
    if (dir)
      ps->setDir(dir);
    ps->drawText(s, m_x0 + x, m_y0 + y, alignment);
    if (dir)
      ps->setDir(0);
  } else
    FrameItem::drawText(painter, s, x, y + 0.01333 * m_relSize * m_width,
                        alignment, dir);
}

void ThreeDItem::frame3D(QPainter* painter, int iBack, PSGraph* ps) const
{
  int i = qBound(1, 2 * iBack - 1, 7);
  double x[5], y[5];
/* the vertices which may be hidden are drawn by a call to plot3D */
  x[0] = vertex[i - 1];
  y[0] = vertex[i];
  x[1] = x[3] = vertex[i + 7];
  y[1] = y[3] = vertex[i + 8];
  i -= 2;
  if (i == -1)
    i = 7;
  x[2] = vertex[i + 7];
  y[2] = vertex[i + 8];
  i += 4;
  if (i > 7)
    i -= 8;
  x[4] = vertex[i + 7];
  y[4] = vertex[i + 8];
  double z = 0;
  QTransform sav = m3D;
  m3D = QTransform();
  plot3D(painter, true, true, false, x, y, &z, 1, 1, 0, false, 3, 0, 0, 0, ps);
  plot3D(painter, true, true, false, &x[3], &y[3], &z, 1, 1, 0, false, 2, 0,
         0, 0, ps);
  m3D = sav;
  x[0] = vertex[i - 1];
  y[0] = vertex[i];
  i -= 2;
  for (int j = 1; j < 4; ++j) {
    i += 2;
    if (i == 9)
      i = 1;
    x[j] = vertex[i + 7];
    y[j] = vertex[i + 8];
  }
  x[4] = vertex[i - 1];
  y[4] = vertex[i];
  plotArray(painter, x, y, 5, ps);
  i -= 2;
  if (i == -1)
    i = 7;
  x[0] = vertex[i - 1];
  y[0] = vertex[i];
  x[1] = vertex[i + 7];
  y[1] = vertex[i + 8];
  plotArray(painter, x, y, 2, ps);
}

void ThreeDItem::line3D(QPainter* painter, double x1, double y1, double z1,
                        double x2, double y2, double z2, PSGraph* ps) const
{
  double x[2], y[2];
  x1 *= m_fX;
  x2 *= m_fX;
  y1 *= m_fY;
  y2 *= m_fY;
  z1 *= m_fZ;
  z2 *= m_fZ;
  x[0] = xFrom3d(x1, z1);
  y[0] = yFrom3d(x1, y1, z1);
  x[1] = xFrom3d(x2, z2);
  y[1] = yFrom3d(x2, y2, z2);
  plotArray(painter, x, y, 2, ps);
}

void ThreeDItem::plotArray3D(QPainter* painter, PSGraph* ps) const
{
  double x = 0.5 * scalx * m_fX * (m_logX ? log10(m_xMax * m_xMin) :
                                            (m_xMax + m_xMin));
  double z = 0.5 * scalz * m_fZ * (m_logZ ? log10(m_zMax * m_zMin) :
                                            (m_zMax + m_zMin));
  xOff3D = m_xRef - m3D.m11() * x - m3D.m13() * z;
  yOff3D = m_yRef - m3D.m21() * x - m3D.m23() * z;
  if (m_mode3D != 1) {
    x = m_fX * scalx * xminl;
    QVector<double> y(nx);
    bool dec = (m3D.m11() * m3D.m22() * zStep > 0);
    for (int nline = 0; nline < nz; ++nline) {
      int lz = dec ? (nz - 1 - nline) : nline;
      z = scalz * m_fZ * (zminl + zStep * lz);
      for (int i = 0; i < nx; ++i) {
        double ya = qBound(m_yMin, m_yValues[i][lz], m_yMax);
        y[i] = m_logY ? log10(ya / m_yMin) : (m_fY * (ya - m_yMin));
      }
      plot3D(painter, false, true, false, &x, y.data(), &z,
             m_fX * xStep * scalx, scaly, 0, nline == 0, nx, xOff3D, yOff3D,
             vertex, ps);
    }
  }
  if (m_mode3D != 0) {
    z = m_fZ * zminl * scalz;
    QVector<double> y(nz);
    bool dec = (m3D.m13() * m3D.m22() * xStep < 0);
    for (int nline = 0; nline < nx; ++nline) {
      int lx = dec ? (nx - 1 - nline) : nline;
      x = scalx * m_fX * (xminl + xStep * lx);
      for (int i = 0; i < nz; ++i) {
        double ya = qBound(m_yMin, m_yValues[lx][i], m_yMax);
        y[i] = m_logY ? log10(ya / m_yMin) : (m_fY * (ya - m_yMin));
      }
      plot3D(painter, false, true, false, &x, y.data(), &z, 0, scaly,
             m_fZ * zStep * scalz, nline == 0, nz, xOff3D, yOff3D, vertex, ps);
    }
  }
}

void ThreeDItem::plot3D(QPainter* painter, bool xArr, bool yArr, bool zArr,
                        const double* xData, const double* yData,
                        const double* zData, double xScale, double yScale,
                        double zScale, bool firstLine, int nPnts, double xOff,
                        double yOff, double* vtx, PSGraph* ps) const
{
  bool xOutLeftOld = false;
  bool xOutRightOld = false;
  bool xOutOld = false;
  bool lowOld = false;
  bool highOld = false;
  int jOld = -1;
  double xOld = 0;
  double yOld = 0;
  QVector<double> xBuf(nPnts);
  QVector<double> yBuf(nPnts);
  int nBuf = 0;
  if (firstLine) {
    yMin3D.fill(DBL_MAX);
    yMax3D.fill(-DBL_MAX);
  }
  for (int i = 0; i < nPnts; ++i) {
    double x3d = xArr ? xData[i] * xScale : xData[0] + i * xScale;
    double y3d = yArr ? yData[i] * yScale : yData[0] + i * yScale;
    double z3d = zArr ? zData[i] * zScale : zData[0] + i * zScale;
    double x = m3D.m11() * x3d + m3D.m13() * z3d + xOff;
    double y0 = m3D.m21() * x3d + m3D.m23() * z3d + yOff;
    double y = y0 + m3D.m22() * y3d;
    bool xOutLeft = x < 0;
    bool xOutRight = x > m_width;
    bool xOut = xOutLeft || xOutRight;
    bool yOut = (y < 0) || (y > m_height);
    bool low, high;
    if (i) { // not first point
      if (!((xOutLeft && xOutLeftOld) || (xOutRight && xOutRightOld))) {
        double xStart = xOutOld ? (xOutLeftOld ? 0 : m_width) : xOld;
        int jStart = qRound(scal3D * xStart);
        int jEnd = xOut ? (xOutLeft ?
                           0 : (nRes3D - 1)) : qRound(scal3D * x);
        if ((fabs(x - xStart) * scal3D) < 0.5)
          jEnd = jStart;
        bool vertical = (jEnd == jStart);
        int mask = 0;
        double yStart = yOld;
        double slope = 1;
        double yMinOld = 0;
        double yMaxOld = 0;
        double ymi1 = 0;
        double yma1 = 0;
        if (vertical) {
          mask = jStart;
          ymi1 = yMinOld = yMin3D.at(mask);
          yma1 = yMaxOld = yMax3D.at(mask);
          jStart = qRound(scal3D * yOld);
          jEnd = qRound(scal3D * y);
        } else {
          slope = (y - yOld) / (x - xOld);
          yStart += slope * (xStart - xOld);
        }
        slope /= scal3D;
        int inc = (jEnd > jStart) ? 1 : -1;
        for (int j = jStart; ; j += inc) {
          double yj = yStart + (j - jStart) * slope;
          if (!vertical) {
            mask = j;
            yMinOld = yMin3D.at(mask);
            yMaxOld = yMax3D.at(mask);
            int k = j - inc;
            ymi1 = yMin3D.at(k);
            yma1 = yMax3D.at(k);
          }
          yMin3D[mask] = qMin(yj, yMinOld);
          yMax3D[mask] = qMax(yj, yMaxOld);
          if ((j == jOld) && (!vertical)) {
            low = lowOld;
            high = highOld;
          } else {
            low = (yj <= yMinOld);
            high = (yj >= yMaxOld);
          }
          if ((low || high) && (!yOut)) { // visible
            if (nBuf) {
              if ((low ^ lowOld) && (high ^ highOld)) {
                if (nBuf > 1) {
                  yBuf[nBuf - 1] = lowOld ? ymi1 : yma1;
                  plotArray(painter, xBuf.data(), yBuf.data(), nBuf, ps);
                  xBuf[0] = xBuf[nBuf - 1];
                  yBuf[0] = low ? ymi1 : yma1;
                }
                nBuf = 1;
              } else {
                if ((nBuf > 1) && (j != jStart)) // replace point
                  nBuf--;
              }
            }
            xBuf[nBuf] = vertical ? x : (xStart + (j - jStart) / scal3D);
            yBuf[nBuf++] = yj;
          } else { // not visible
            if (nBuf > 1) {
              yBuf[nBuf - 1] = lowOld ? ymi1 : yma1;
              plotArray(painter, xBuf.data(), yBuf.data(), nBuf, ps);
            }
            nBuf = 0;
          }
          lowOld = low;
          highOld = high;
          jOld = mask;
          if (j == jEnd)
            break;
        }
      }
      if (vtx) {
        vtx[4] = vtx[12] = x;
        vtx[5] = y;
        vtx[13] = y0;
        if (firstLine) {
          vtx[2] = vtx[10] = x;
          vtx[3] = y;
          vtx[11] = y0;
        }
      }
    } else { // first point
      if (vtx) {
        vtx[6] = vtx[14] = x;
        vtx[7] = y;
        vtx[15] = y0;
        if (firstLine) {
          vtx[0] = vtx[8] = x;
          vtx[1] = y;
          vtx[9] = y0;
        }
      }
    }
    xOld = x;
    yOld = y;
    xOutLeftOld = xOutLeft;
    xOutRightOld = xOutRight;
    xOutOld = xOut;
  }
  if (nBuf > 1)
    plotArray(painter, xBuf.data(), yBuf.data(), nBuf, ps);
}

void ThreeDItem::plotArray(QPainter* painter, const double* x, const double* y,
                           int n, PSGraph* ps) const
{
  if (ps)
    ps->drawArray(x, y, 1, 1, n);
  else {
    QPolygonF ap;
    for (int i = 0; i < n; ++i)
      ap << md.map(QPointF(x[i], y[i]));
    if (painter) {
      painter->setClipRect(clipRect());
      painter->drawPolyline(ap);
      if (scene())
        painter->setClipRect(static_cast<KplScene*>(scene())->sceneRect());
      else qDebug("ThreeDItem::plotArray(): no scene!");
    }
  }
}

void ThreeDItem::raster3d(QPainter* painter, PSGraph* ps) const
{
  if (ps) {
    ps->setRelSize(1.414 * m_relSize);
    ps->setColorData(m_colorFrame);
  }
  frame3D(painter, (m3D.m11() * m3D.m22() < 0) ? 3 : 4, ps);
  if (m_gridMode) {
    double xa, za;
    int ix, iz;
    xa = xminl;
    double xe = xa + (nx - 1) * xStep;
    if (m3D.m21() * m3D.m22() * xStep < 0) {
      xa = xe * scalx;
      ix = -1;
    } else {
      xa *= scalx;
      ix = 1;
    }
    za = zminl;
    double ze = za + (nz - 1) * zStep;
    if (m3D.m23() * m3D.m22() * zStep < 0) {
      za = ze * scalz;
      iz = -1;
    } else {
      za *= scalz;
      iz = 1;
    }
    double xl = scalx * ((m3D.m11() < 0) ? xe : xminl);
    double zl = scalz * ((m3D.m13() < 0) ? ze : zminl);
    line3D(painter, xl, 0, zl, xl,
           (m_logY ? log10(m_yMax / m_yMin) : (m_yMax - m_yMin)) * scaly, zl,
           ps);
    double dx1 = 0.04 * nx * xStep * scalx;
    double dx2 = ix * dx1;
    double a = m3D.m11() * m3D.m11() + m3D.m21() * m3D.m21();
    if (a > 0)
      dx2 /= sqrt(a);
    double dz2 = 0.04 * iz * nz * zStep * scalz;
    a = m3D.m13() * m3D.m13() + m3D.m23() * m3D.m23();
    if (a > 0)
      dz2 /= sqrt(a);
    if (painter) {
      QPen p = pen();
      p.setColor(m_colorGrid);
      painter->setPen(changePenWidth(p, p.widthF() * penWidthFactor(GridPen) /
                      penWidthFactor(FramePen)));
    }
    if (ps)
      ps->setColorData(m_colorGrid);
    axis3D(painter, m_xMin, m_xMax, m_xTic, m_minTicX, m_fX, scalx, xa, za,
           dz2, m_numDigX, X, m_logX, ps);
    axis3D(painter, m_zMin, m_zMax, m_zTic, m_minTicZ, m_fZ, scalz, xa, za,
           dx2, m_numDigZ, Z, m_logZ, ps);
    axis3D(painter, m_yMin, m_yMax, m_yTic, m_minTicY, m_fY, scaly, m_fX * xl,
           m_fZ * zl, m_fX * dx1,
           m_numDigY, Y, m_logY, ps);
  }
  if (ps)
    ps->setRelSize(m_relSize);
}

void ThreeDItem::setProperties()
{
  xminl = m_logX ? log10(m_xMin) : m_xMin;
  zminl = m_logZ ? log10(m_zMin) : m_zMin;
  QRectF r = m_matrix.mapRect(QRectF(m_x0, m_y0, m_width, m_height));
  setRect(r);
  QPen pen = createPen(this, m_colorFrame, FramePen);
  double w = r.width();
  double d = w * m_relSize * penWidthFactor(FramePen);
  setPen(changePenWidth(pen, d));
  md = m_matrix;
  md.translate(m_x0, m_y0);
  d *= 0.5;
  cRect = r.adjusted(d, d, -d, -d);
}

double ThreeDItem::xFrom3d(double x, double z) const
{
  return m3D.m11() * x + m3D.m13() * z + xOff3D;
}

double ThreeDItem::yFrom3d(double x, double y, double z) const
{
  return m3D.m21() * x + m3D.m23() * z + m3D.m22() * y + yOff3D;
}
