/***************************************************************************
                          utils.h  -  description
                             -------------------
    begin                : Sun Jan 7 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef UTILS_H
#define UTILS_H

class QLibrary;
class QString;
class QWidget;
class QUrl;

/**
  * Utility namespace. Provides utility functions in a namespace.
  * @author Werner Stille
  */
namespace Utils {
  /**
   * Downloads file.
   * @param src source file.
   * @param target local destination.
   * @param window parent widget.
   * @return true for success.
   */
  Q_DECL_EXPORT bool download(const QUrl& src, QString& target, QWidget* window = 0);
  /**
   * Generates sorted list of function names.
   * @param libPath path of function library.
   * @return list of names.
   */
  Q_DECL_EXPORT QStringList funcNames(const QString& libPath);
  /**
   * Gets module handle and address of a shared library function.
   * @param path path of the library.
   * @param name function name.
   * @param lib pointer to library.
   * @param fkt pointer to function address.
   */
  Q_DECL_EXPORT bool getFuncAddr(const QString& path, const QString& name,
                                 QLibrary** lib, QFunctionPointer* fkt);
  /**
   * Moves file to remote location.
   * @param src temporary source file.
   * @param dest remote destination.
   * @param window parent widget.
   * @return true for success.
   */
  Q_DECL_EXPORT bool move(const QString& src, const QUrl& dest, QWidget* window = 0);
  /**
   * Generates unique temporary file name.
   * @param suffix filename suffix.
   * @return full path of temporary file.
   */
  Q_DECL_EXPORT QString tempFileName(const QString& suffix = ".tmp");
  /**
   * Returns URL for given user input, assuming local file.
   * @param s user input string.
   */
  Q_DECL_EXPORT QUrl url(const QString& s);
}

#endif
