/***************************************************************************
                          brushdlg.h  -  description
                             -------------------
    begin                : Mit Mai 14 2003
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BRUSHDLG_H
#define BRUSHDLG_H

#include <QPushButton>
#include <QRadioButton>
#include <QDialog>

class QButtonGroup;

/**
  * Brush button. Provides a button which shows the current
  * brush fill pattern and opens a dialog to select a pattern.
  * @author Werner Stille
  */
class BrushButton : public QPushButton
{
  Q_OBJECT
  Q_PROPERTY(int value READ value WRITE setValue USER true )

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param brush pattern number.
   */
  BrushButton(int brush = 0, QWidget* parent = 0);
  /** Destructor. */
  ~BrushButton();
  /**
   * Sets pattern number.
   * @param brush pattern number.
   */
  void setValue(int brush);
  /** Reimplemented from QPushButton. */
  virtual QSize sizeHint() const;
  /**
   * Returns pattern number.
   * @return pattern number.
   */
  int value();

protected slots:
  /** Displays the dialog for selection of a pattern. */
  void slotBrush();

signals:
  /**
   * Emitted when brush changed.
   * @param brush pattern number.
   */
  void changed(int brush);

protected:
  /** Reimplemented from QPushButton. */
  virtual void paintEvent(QPaintEvent* event);
  int m_brush;
};

/**
  * Brush radio button. Provides a radio button which shows a
  * brush fill pattern.
  * @author Werner Stille
  */
class BrushRadioButton : public QRadioButton
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param brush pattern number.
   * @param parent pointer to parent widget.
   */
  BrushRadioButton(int brush, QWidget* parent = 0);
  /** Destructor. */
  ~BrushRadioButton();

protected:
  /** Reimplemented from QRadioButton. */
  virtual void paintEvent(QPaintEvent* event);
  int m_brush;
};

/**
  * Brush dialog. Provides a dialog to select a brush fill pattern.
  * @author Werner Stille
  */
class BrushDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param brush pointer to pattern number.
   * @param parent pointer to parent widget.
   */
  BrushDlg(int* brush, QWidget* parent = 0);
  /** Destructor. */
  ~BrushDlg();

protected slots:
  /** Reimplemented from QDialog. */
  virtual void accept();
  /** Enables Ok button. */
  void slotEdited();

protected:
  int* m_brush;
  QButtonGroup* bg;
  QPushButton* bOk;
};

#endif
