/***************************************************************************
                          decodlg.cpp  -  description
                             -------------------
    begin                : Fri Apr 20 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QDialogButtonBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QGridLayout>
#include <QKeyEvent>
#include <QLabel>
#include <KColorButton>
#include <KLocalizedString>
#include "decodlg.h"
#include "decoitem.h"
#include "kpldoc.h"
#include "spinbox.h"

DecoDlg::DecoDlg(QWidget* parent, const QString& caption, KplDoc* model,
                 const DecoItem* it0, const QPoint& pos) :
 QDialog(parent), m(model), it(it0)
{
  setWindowTitle(caption);
  vbox = new QVBoxLayout(this);
  vbox->addLayout(grid = new QGridLayout);
  vbox->addWidget(buttonBox = new QDialogButtonBox(QDialogButtonBox::Help |
                                                   QDialogButtonBox::Ok |
                                                   QDialogButtonBox::Apply |
                                                   QDialogButtonBox::Cancel));
  resize(minimumSize());
  move(pos);
}

DecoDlg::~DecoDlg()
{
}

void DecoDlg::initEntries(int row)
{
  grid->addWidget(new QLabel("x0"), row, 0);
  grid->addWidget(ex0 = new QDoubleSpinBox, row, 1);
  ex0->setRange(-200, 200);
  ex0->setSingleStep(0.1);
  ex0->setSuffix(" cm");
  ex0->setValue(it->x0());
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row,
                2);
  grid->addWidget(new QLabel("y0"), row, 3);
  grid->addWidget(ey0 = new QDoubleSpinBox, row++, 4);
  ey0->setRange(-200, 200);
  ey0->setSingleStep(0.1);
  ey0->setSuffix(" cm");
  ey0->setValue(it->y0());
  grid->addWidget(lSize = new QLabel(i18n("Size")), row, 0);
  grid->addWidget(eRelSize = new SpinBox(qRound(100 * it->relSize()), 10, 800),
                  row, 1);
  eRelSize->setSuffix(" %");
  grid->addWidget(new QLabel(i18n("Color")), row, 3);
  grid->addWidget(col = new KColorButton(it->color()), row, 4);
  buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
  buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
  connect(ex0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(ey0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(col, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(eRelSize, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
}

void DecoDlg::getValues(DecoItem* itNew)
{
  itNew->setX0(ex0->value());
  itNew->setY0(ey0->value());
  itNew->setRelSize(0.01 * eRelSize->value());
  itNew->setColor(col->color().rgb());
}

void DecoDlg::keyPressEvent(QKeyEvent* e)
{
  if ((e->QInputEvent::modifiers() == Qt::NoModifier) &&
      (e->key() == Qt::Key_F1)) {
    buttonBox->button(QDialogButtonBox::Help)->animateClick();
    e->accept();
  } else
    QDialog::keyPressEvent(e);
}

void DecoDlg::slotEdited()
{
  buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);
  buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
}
