/***************************************************************************
                          doubleedit.h  -  description
                             -------------------
    begin                : Tue Aug 15 2000
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOUBLEEDIT_H
#define DOUBLEEDIT_H

#include <QLineEdit>

class QDoubleValidator;

/**
  * Double value line edit class. Provides a QLineEdit widget with validator.
  * @author Werner Stille
  */
class DoubleEdit : public QLineEdit
{
  Q_OBJECT
  Q_PROPERTY(double value READ value WRITE setValue USER true )

public:
  /**
   * Constructor.
   * @param val initial double value.
   * @param parent pointer to parent widget.
   * @param format format character for value display.
   * @param prec precision (number of digits) for value display.
   */
  DoubleEdit(double val, char format = 'g', int prec = 6, QWidget* parent = 0);
  /**
   * Constructor.
   * @param val initial double value.
   * @param minVal minimum value.
   * @param maxVal maximum value.
   * @param parent pointer to parent widget.
   * @param format format character for value display.
   * @param prec precision (number of digits) for value display.
   */
  DoubleEdit(double val, double minVal, double maxVal, char format = 'f',
             int prec = 2, QWidget* parent = 0);
  /** Destructor. */
  virtual ~DoubleEdit();
  /**
   * Fixups and sets value.
   * @param val double value.
   */
  void setValue(double val);
  /**
   * Fixups and returns value.
   * @return value.
   */
  double value() const;
  /**
   * Sets lower limit of validator.
   * @param bottom lower limit.
   */
  void setBottom(double bottom);

protected:
  /** Reimplemented from QLineEdit for fixup of text. */
  virtual void focusOutEvent(QFocusEvent*);
  /** Interprets text. */
  void interpret();
  char frm;
  QDoubleValidator* validator;

signals:
  /**
   * Emitted when value changes.
   * @param val new value.
   */
  void changed(double val);
};

#endif
