/***************************************************************************
                          ellipsedlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QCheckBox>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QLayout>
#include <QUndoStack>
#include <KLocalizedString>
#include "brushdlg.h"
#include "kpldoc.h"
#include "ellipseitem.h"
#include "ellipsedlg.h"
#include "spinbox.h"
#include "symboldlg.h"

EllipseDlg::EllipseDlg(QWidget* parent, KplDoc* model, EllipseItem* ei0,
                       const QPoint& pos) :
 DecoDlg(parent, i18n("Ellipse"), model, ei0, pos), ei(ei0)
{
  grid->addWidget(new QLabel(i18n("Width")), 0, 0);
  grid->addWidget(eWidth = new QDoubleSpinBox, 0, 1);
  eWidth->setRange(0.01, 200);
  eWidth->setSingleStep(0.1);
  eWidth->setSuffix(" cm");
  eWidth->setValue(ei->width());
  connect(eWidth, SIGNAL(valueChanged(const QString&)), SLOT(slotAspect()));
  grid->addWidget(new QLabel(i18n("Height")), 0, 3);
  grid->addWidget(eHeight = new QDoubleSpinBox, 0, 4);
  eHeight->setRange(0.01, 200);
  eHeight->setSingleStep(0.1);
  eHeight->setSuffix(" cm");
  eHeight->setValue(ei->height());
  grid->addWidget(isotropic = new QCheckBox(i18n("Circle")), 1, 1, 1, 5);
  connect(isotropic, SIGNAL(toggled(bool)), SLOT(slotAspect()));
  isotropic->setChecked(ei->width() == ei->height());
  grid->addWidget(new QLabel(i18n("Rotation")), 2, 0);
  grid->addWidget(eDir = new QDoubleSpinBox, 2, 1);
  eDir->setRange(-360, 360);
  eDir->setSuffix(QString::fromUtf8("\302\260"));
  eDir->setValue(ei->dir());
  initEntries(3);
  grid->addWidget(new QLabel(i18n("Type")), 5, 0);
  grid->addWidget(symb = new SpinBox(ei->symbol(), 1, 9), 5, 1);
  grid->addWidget(bSymb = new SymbolButton(symb, true, false), 5, 2);
  eRelSize->setDisabled(ei->fillStyle());
  lSize->setText(i18n("Line width"));
  grid->addWidget(new QLabel(i18n("Fill pattern")), 5, 3);
  grid->addWidget(fill = new BrushButton(ei->fillStyle()), 5, 4);
  connect(fill, SIGNAL(changed(int)), SLOT(slotFillChanged(int)));
  connect(eHeight, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(symb, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
}

EllipseDlg::~EllipseDlg()
{
}

void EllipseDlg::slotAspect()
{
  bool iso = isotropic->isChecked();
  if (iso)
    eHeight->setValue(eWidth->value());
  eHeight->setDisabled(iso);
  slotEdited();
}

void EllipseDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
      buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
      buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    case QDialogButtonBox::Ok: {
      EllipseItem* eiNew = new EllipseItem(*ei);
      getValues(eiNew);
      double w = eWidth->value();
      eiNew->setWidth(w);
      eiNew->setHeight(isotropic->isChecked() ? w : eHeight->value());
      eiNew->setDir(eDir->value());
      eiNew->setSymbol(KplItem::Symbol(symb->value()));
      eiNew->setFillStyle(KplItem::FillStyle(fill->value()));
      m->undoStack()->push(new EditCommand(eiNew, ei, m));
      ei = eiNew;
      if (id == QDialogButtonBox::Ok)
        accept();
      break;
    }
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/"
                                                    "ellipse.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}

void EllipseDlg::slotFillChanged(int brush)
{
  symb->setDisabled(brush);
  bSymb->setDisabled(brush);
  eRelSize->setDisabled(brush);
  slotEdited();
}
