/***************************************************************************
                          errmoddlg.cpp  -  description
                             -------------------
    begin                : Tue May 25 1999
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QButtonGroup>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QKeyEvent>
#include <QLabel>
#include <QLayout>
#include <QLibrary>
#include <QPushButton>
#include <QRadioButton>
#include <KLocalizedString>
#include <KUrlRequester>
#include "choosefuncdlg.h"
#include "doubleedit.h"
#include "editparameterdlg.h"
#include "errmoddlg.h"
#include "funcdlg.h"
#include "kpldoc.h"
#include "kplsettings.h"
#include "kplutils.h"
#include "utils.h"

ErrModDlg::ErrModDlg(QWidget* parent, KplDoc* model,
                     FunItem::DataErrorStruct* err0) :
 QDialog(parent),
 m(model), err(err0)
{
  errt = new FunItem::DataErrorStruct(*err);
  setWindowTitle(i18n("Error model function"));
  QVBoxLayout* vbox = new QVBoxLayout(this);
  QGridLayout* grid = new QGridLayout;
  vbox->addLayout(grid);
  grid->addWidget(new QLabel(i18n("Library")), 0, 0);
  grid->addWidget(fileName =
                  new KUrlRequester(Utils::url(err->errModPath)),
                                    0, 1);
  fileName->setStartDir(Utils::url(KplSettings::currentDir()));
  fileName->setFilter("*.so\n*");
  fileName->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
  grid->addWidget(new QLabel(i18n("Function")), 1, 0);
  QHBoxLayout* hbox = new QHBoxLayout;
  grid->addLayout(hbox, 1, 1);
  hbox->addWidget(func = new QLineEdit(err->errModName));
  QPushButton* tb = new QPushButton("...");
  tb->setFixedSize(fileName->button()->size());
  hbox->addWidget(tb);
  connect(tb, SIGNAL(clicked()), SLOT(slotFunc()));
  grid->addWidget(new QLabel(i18n("Parameter")), 2, 0);
  grid->addLayout(hbox = new QHBoxLayout, 2, 1);
  QPushButton* b;
  hbox->addWidget(b = new QPushButton(i18n("Edit")));
  connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Load")));
  connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Save")));
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
  grid->addWidget(new QLabel(i18n("Argument")), 3, 0);
  grid->addLayout(hbox = new QHBoxLayout(), 3, 1, 1, 3);
  bg = new QButtonGroup;
  QRadioButton* rb;
  hbox->addWidget(rb = new QRadioButton(i18n("x column")));
  bg->addButton(rb, 0);
  hbox->addWidget(rb = new QRadioButton(i18n("y column")));
  bg->addButton(rb, 1);
  bg->button(err->errModArg)->setChecked(true);
  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::Expanding));
  vbox->addWidget(buttonBox = new QDialogButtonBox(QDialogButtonBox::Help |
                                  QDialogButtonBox::Ok |
                                  QDialogButtonBox::Cancel));
  bOk = buttonBox->button(QDialogButtonBox::Ok);
  bOk->setFocus();
  bOk->setEnabled(false);
  connect(fileName, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(func, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(bg, SIGNAL(buttonClicked(int)), SLOT(slotEdited()));
  connect(buttonBox->button(QDialogButtonBox::Help), SIGNAL(clicked()),
          SLOT(slotHelp()));
  connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
  connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
}

ErrModDlg::~ErrModDlg()
{
  delete errt;
}

void ErrModDlg::keyPressEvent(QKeyEvent* e)
{
  if ((e->QInputEvent::modifiers() == Qt::NoModifier) &&
      (e->key() == Qt::Key_F1)) {
    buttonBox->button(QDialogButtonBox::Help)->animateClick();
    e->accept();
  } else
    QDialog::keyPressEvent(e);
}

void ErrModDlg::accept()
{
  errt->errModPath = fileName->url().path();
  errt->errModName = func->text();
  errt->errModArg =
    FunItem::DataErrorStruct::ErrorModelArgument(bg->checkedId());
  QLibrary* lib;
  double (*fkt)(double, const double *);
  if (!Utils::getFuncAddr(errt->errModPath, errt->errModName, &lib,
                          (QFunctionPointer*) &fkt))
    return;
  lib->unload();
  delete lib;
  *err = *errt;
  QDialog::accept();
}

void ErrModDlg::slotEdited()
{
  bOk->setEnabled(true);
}

void ErrModDlg::slotEditParameter()
{
  EditParameterDlg dlg(this, errt->parErrMod, fileName->url().path(),
                       func->text());
  if (dlg.exec())
    slotEdited();
}

void ErrModDlg::slotFunc()
{
  KplUtils::getFunc(this, fileName->url(), func);
}

void ErrModDlg::slotGetParameter()
{
  if (m->getPar(this, errt->parErrMod))
    slotEdited();
}

void ErrModDlg::slotHelp()
{
  QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/fit.html"
                                                "#errormodel")));
}

void ErrModDlg::slotSaveParameter()
{
  m->saveFunPar(this, errt->parErrMod);
}
