/***************************************************************************
                          errmoddlg.h  -  description
                             -------------------
    begin                : Tue May 25 1999
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ERRMODDLG_H
#define ERRMODDLG_H

#include <QDialog>
#include "funitem.h"

class QDialogButtonBox;
class QLineEdit;
class QButtonGroup;
class KUrlRequester;
class KplDoc;

/**
  * Error model dialog class. Provides a dialog for setting
  * the calculation of the error bars for nonlinear fits.
  * @author Werner Stille
  */
class ErrModDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param err0 pointer to structure containing the data error settings.
   */
  ErrModDlg(QWidget* parent, KplDoc* model, FunItem::DataErrorStruct* err0);
  /** Destructor. */
  ~ErrModDlg();

protected:
  /**
   * Reimplemented to display help when key F1 is pressed.
   * @param e Key event.
   */
  virtual void keyPressEvent(QKeyEvent* e);
  KplDoc* m;
  FunItem::DataErrorStruct *err, *errt;
  KUrlRequester* fileName;
  QLineEdit* func;
  QButtonGroup *bg;
  QDialogButtonBox* buttonBox;
  QPushButton* bOk;

protected slots:
  /** Reimplemented from QDialog. */
  virtual void accept();
  /** Enables buttons. */
  void slotEdited();
  /** Displays the dialog for parameter editing. */
  void slotEditParameter();
  /** Displays the dialog for selection of a function. */
  void slotFunc();
  /** Displays the dialog for selection of a parameter file to read. */
  void slotGetParameter();
  /** Displays help page. */
  void slotHelp();
  /** Displays the dialog for selection of a parameter file to save. */
  void slotSaveParameter();
};

#endif
