/***************************************************************************
                          framedlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FRAMEDLG_H
#define FRAMEDLG_H

#include <QDialog>

class QButtonGroup;
class QCheckBox;
class QDialogButtonBox;
class QDoubleSpinBox;
class QLineEdit;
class QPushButton;
class QUndoCommand;
class KColorButton;
class DoubleEdit;
class FrameItem;
class KplDoc;
class SpinBox;

/**
  * Frame settings dialog class. Provides a dialog for the
  * settings of frame and labels.
  * @author Werner Stille
  */
class FrameDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param fi pointer to frame item.
   * @param pos dialog position.
   */
  FrameDlg(QWidget* parent, KplDoc* model, FrameItem* fi, const QPoint& pos);
  /** Destructor. */
  ~FrameDlg();

protected:
  /**
   * Reimplemented to display help when key F1 is pressed.
   * @param e Key event.
   */
  virtual void keyPressEvent(QKeyEvent* e);
  /** Normalizes all child items of the frame item. */
  void normalize() const;
  KplDoc* m;
  bool norm;
  double fx, fy;
  FrameItem* fi;
  DoubleEdit *exMin, *exMax, *eyMin, *eyMax, *exTic, *eyTic, *exOff, *eyOff;
  QDoubleSpinBox *ex0, *exl, *ey0, *eyl;
  QLineEdit *eLetX, *eLetY, *eLetH;
  QButtonGroup* bg;
  QPushButton* bOk;
  QDialogButtonBox* buttonBox;
  KColorButton *colFrame, *colGrid;
  SpinBox *emTicx, *emTicy, *enDigx, *enDigy, *ex, *ey, *eRelSiz;
  QCheckBox *logX, *logY, *xBottom, *xTop, *yLeft, *yRight, *autoNorm;
  mutable QUndoCommand* macro;

protected slots:
  /**
   * Performs automatic scaling of arrays and functions to be
   * plotted in this frame.
   */
  void slotAutoScale();
  /**
   * Handles button clicks.
   * @param button pointer to button.
   */
  void slotButtonClicked(QAbstractButton* button);
  /** Enables buttons. */
  void slotEdited();
  /** Updates dialog according to logarithmic axis option. */
  void slotLogToggled(bool state);
  /** Updates dialog according to autonormalization option. */
  void slotNormToggled(bool state);
  /** Displays dialog to edit the heading. */
  void slotEditLetH();
  /** Displays dialog to edit the x axis label. */
  void slotEditLetX();
  /** Displays dialog to edit the y axis label. */
  void slotEditLetY();
  /** Restores window size. */
  void slotRestoreSize();
};

#endif
