/***************************************************************************
                          utils.cpp  -  description
                             -------------------
    begin                : Sun Jan 7 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QFileDialog>
#include <QFileInfo>
#include <QLineEdit>
#include <QUrl>
#include <KIO/Job>
#include <KLocalizedString>
#include <KMessageBox>
#include "choosefuncdlg.h"
#include "kplsettings.h"
#include "kplutils.h"
#include "utils.h"

ArrayItem::DecimalCharacter KplUtils::decimalCharacter()
{
  return static_cast<ArrayItem::DecimalCharacter>(
    KplSettings::decimalCharacter());
}

void KplUtils::getFunc(QWidget* parent, const QUrl& url, QLineEdit* func)
{
  QStringList names = Utils::funcNames(url.path());
  if (names.isEmpty())
    KMessageBox::error(parent,
                       i18n("while trying to open module definition file"));
  else {
    ChooseFuncDlg dlg(parent, names, func);
    dlg.exec();
  }
}

bool KplUtils::getWriteUrl(QWidget* parent, QUrl& u, const QString& startDir,
                           const QString& filter)
{
  u = QFileDialog::getSaveFileUrl(parent, QString(), Utils::url(startDir),
                                  filter);
  bool success = !u.isEmpty();
  if (success) {
    bool exists = false;
    bool writable = true;
    if (u.isLocalFile()) {
      QFileInfo fi(u.path());
      exists = fi.exists();
      if (exists)
        writable = fi.isWritable();
    } else {
      KIO::StatJob* job = KIO::stat(u, KIO::StatJob::StatSide::SourceSide, 0);
      exists = job->exec();
      job->deleteLater();
      if (exists) {
        job = KIO::stat(u, KIO::StatJob::StatSide::DestinationSide, 0);
        writable = job->exec();
        job->deleteLater();
      }
    }
    if (exists && (!writable)) {
      KMessageBox::sorry(parent, i18n("Writing to this file not possible."));
      success = false;
    }
  }
  return success;
}

void KplUtils::setCurrentDir(const QUrl& url)
{
  QFileInfo fi(url.path());
  QUrl u = url;
  u.setPath(fi.absolutePath());
  QString s = u.toString();
  if (!s.endsWith("/"))
    s += "/";
  KplSettings::setCurrentDir(s);
}
