/***************************************************************************
                          utils.h  -  description
                             -------------------
    begin                : Sun Jan 7 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPLUTILS_H
#define KPLUTILS_H

#include "arrayitem.h"

class QLineEdit;
class QString;
class QWidget;
class QUrl;

/**
  * Utility namespace. Provides utility functions in a namespace.
  * @author Werner Stille
  */
namespace KplUtils {
  /**
   * Returns decimal symbol of floating point numbers used in data files.
   * @return decimal character.
   */
  ArrayItem::DecimalCharacter decimalCharacter();
  /**
   * Opens a function selection dialog.
   * @param parent parent widget.
   * @param name URL of the library.
   * @param func QLineEdit widget to display the function name.
   */
  void getFunc(QWidget* parent, const QUrl& url, QLineEdit* func);
  /**
   * Retrieves URL for saving.
   * @param parent parent widget.
   * @param url QUrl receiving the chosen file URL.
   * @param startDir starting directory.
   * @param filter file name filter for file dialog.
   * @return true when file can be saved.
   */
  bool getWriteUrl(QWidget* parent, QUrl& url,
                   const QString& startDir = QString(),
                   const QString& filter = "*.dat (*.dat);;* (*)");
  /**
   * Sets the current directory path.
   * @param url current path.
   */
  void setCurrentDir(const QUrl& url);
}

#endif
