/***************************************************************************
                          linedlg.cpp  -  description
                             -------------------
    begin                : Mon Apr 16 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QLayout>
#include <QLabel>
#include <QUndoStack>
#include <KLocalizedString>
#include "arrowdlg.h"
#include "kpldoc.h"
#include "linedlg.h"
#include "lineitem.h"
#include "spinbox.h"
#include "symboldlg.h"

LineDlg::LineDlg(QWidget* parent, KplDoc* model, LineItem* ld0,
                 const QPoint& pos) :
 DecoDlg(parent, i18n("Line"), model, ld0, pos), li(ld0)
{
  ArrowDlg::initPolarEntries(li, grid, &eDir, &eLen);
  initEntries(1);
  lSize->setText(i18n("Width"));
  grid->addWidget(new QLabel(i18n("Type")), 3, 0);
  grid->addWidget(symb = new SpinBox(li->symbol(), 1, 9), 3, 1);
  grid->addWidget(new SymbolButton(symb, true, false), 3, 2);
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eLen, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(symb, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
}

LineDlg::~LineDlg()
{
}

void LineDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
      buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
      buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    case QDialogButtonBox::Ok: {
      LineItem* liNew = new LineItem(*li);
      liNew->setSymbol(KplItem::Symbol(symb->value()));
      liNew->setDir(eDir->value());
      liNew->setLength(eLen->value());
      getValues(liNew);
      m->undoStack()->push(new EditCommand(liNew, li, m));
      li = liNew;
      if (id == QDialogButtonBox::Ok)
        accept();
      break;
    }
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/line.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}
