//
// C++ Interface: newitemdlg
//
// Description: 
//
//
// Author: Werner Stille <kpl@werner-stille.de>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef NEWITEMDLG_H
#define NEWITEMDLG_H

#include <QDialog>

class QButtonGroup;
class QGridLayout;
// class QGroupBox;

/**
  * New item dialog class.
  * Provides a dialog to choose the type of a new item.
  * @author Werner Stille
  */
class NewItemDlg : public QDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param type pointer to chosen plot item type.
   * @param pos dialog position.
   */
  NewItemDlg(QWidget* parent, int* type, const QPoint& pos = QPoint());
  /** Destructor. */
  ~NewItemDlg();

protected slots:
  /**
   * Accepts selection and leaves the dialog.
   * @param id chosen plot item type.
   */
  void slotClicked(int id);
  /** Displays help page. */
  void slotHelp();

protected:
  /**
   * Adds pushbutton for item type selection.
   * @param text button text.
   * @param id chosen plot item type.
   */
  void addButton(const QString& text, int id);
  /**
   * Reimplemented to display help when key F1 is pressed.
   * @param e Key event.
   */
  virtual void keyPressEvent(QKeyEvent* e);
  QGridLayout* grid;
  QButtonGroup* bg;
  QDialogButtonBox* buttonBox;
  int* m_type;
  int iPos, maxWidth;
};

#endif
