/***************************************************************************
                          rectdlg.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QCheckBox>
#include <QDesktopServices>
#include <QDialogButtonBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QLayout>
#include <QUndoStack>
#include <KLocalizedString>
#include "brushdlg.h"
#include "kpldoc.h"
#include "rectitem.h"
#include "rectdlg.h"
#include "spinbox.h"
#include "symboldlg.h"

RectDlg::RectDlg(QWidget* parent, KplDoc* model, RectItem* ri0,
                 const QPoint& pos) :
 DecoDlg(parent, i18n("Rectangle"), model, ri0, pos), ri(ri0)
{
  grid->addWidget(new QLabel(i18n("Width")), 0, 0);
  grid->addWidget(eWidth = new QDoubleSpinBox, 0, 1);
  eWidth->setRange(0.01, 200);
  eWidth->setSingleStep(0.1);
  eWidth->setSuffix(" cm");
  eWidth->setValue(ri->width());
  connect(eWidth, SIGNAL(valueChanged(const QString&)), SLOT(slotAspect()));
  grid->addWidget(new QLabel(i18n("Height")), 0, 3);
  grid->addWidget(eHeight = new QDoubleSpinBox, 0, 4);
  eHeight->setRange(0.01, 200);
  eHeight->setSingleStep(0.1);
  eHeight->setSuffix(" cm");
  eHeight->setValue(ri->height());
  grid->addWidget(isotropic = new QCheckBox(i18n("Square")), 1, 1, 1, 5);
  connect(isotropic, SIGNAL(toggled(bool)), SLOT(slotAspect()));
  isotropic->setChecked(ri->width() == ri->height());
  grid->addWidget(new QLabel(i18n("Rotation")), 2, 0);
  grid->addWidget(eDir = new QDoubleSpinBox, 2, 1);
  eDir->setRange(-360, 360);
  eDir->setSuffix(QString::fromUtf8("\302\260"));
  eDir->setValue(ri->dir());
  initEntries(3);
  grid->addWidget(new QLabel(i18n("Type")), 5, 0);
  grid->addWidget(symb = new SpinBox(ri->symbol(), 1, 9), 5, 1);
  grid->addWidget(bSymb = new SymbolButton(symb, true, false), 5, 2);
  eRelSize->setDisabled(ri->fillStyle());
  lSize->setText(i18n("Line width"));
  grid->addWidget(new QLabel(i18n("Fill pattern")), 5, 3);
  grid->addWidget(fill = new BrushButton(ri->fillStyle()), 5, 4);
  connect(fill, SIGNAL(changed(int)), SLOT(slotFillChanged(int)));
  connect(eHeight, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(symb, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
}

RectDlg::~RectDlg()
{
}

void RectDlg::slotAspect()
{
  bool iso = isotropic->isChecked();
  if (iso)
    eHeight->setValue(eWidth->value());
  eHeight->setDisabled(iso);
  slotEdited();
}

void RectDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
      buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
      buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
    case QDialogButtonBox::Ok: {
      RectItem* riNew = new RectItem(*ri);
      getValues(riNew);
      double w = eWidth->value();
      riNew->setWidth(w);
      riNew->setHeight(isotropic->isChecked() ? w : eHeight->value());
      riNew->setDir(eDir->value());
      riNew->setSymbol(KplItem::Symbol(symb->value()));
      riNew->setFillStyle(KplItem::FillStyle(fill->value()));
      m->undoStack()->push(new EditCommand(riNew, ri, m));
      ri = riNew;
      if (id == QDialogButtonBox::Ok)
        accept();
      break;
    }
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/rect.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}

void RectDlg::slotFillChanged(int brush)
{
  symb->setDisabled(brush);
  bSymb->setDisabled(brush);
  eRelSize->setDisabled(brush);
  slotEdited();
}
