/***************************************************************************
                          refreshdlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QCheckBox>
#include <QDialogButtonBox>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QTimer>
#include <KLocalizedString>
#include "spinbox.h"
#include "refreshdlg.h"

RefreshDlg::RefreshDlg(QWidget* parent, QTimer* t) : QDialog(parent), timer(t)
{
  setWindowTitle(i18n("Reload periodically"));
  QVBoxLayout* vbox = new QVBoxLayout(this);
  QHBoxLayout* hbox = new QHBoxLayout;
  vbox->addLayout(hbox);
  hbox->addWidget(new QLabel(i18n("Interval")));
  hbox->addWidget(dt = new SpinBox(timer->interval() / 1000, 1, 600));
  dt->setSuffix(" s");
  bool b = timer->isActive();
  dt->setEnabled(b);
  hbox->addWidget(activate = new QCheckBox(i18n("Activate")));
  activate->setChecked(b);
  vbox->addWidget(buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok |
                                                   QDialogButtonBox::Cancel));
  buttonBox->button(QDialogButtonBox::Ok)->setFocus();
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
  connect(activate, SIGNAL(toggled(bool)), dt, SLOT(setEnabled(bool)));
}

RefreshDlg::~RefreshDlg()
{
}

void RefreshDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Ok:
      timer->setInterval(1000 * dt->value());
      if (activate->isChecked())
        timer->start();
      else
        timer->stop();
      accept();
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}
