/***************************************************************************
                          residualsdlg.cpp  -  description
                             -------------------
    begin                : Sun Jul 15 2001
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QApplication>
#include <QDesktopWidget>
#include <QDialogButtonBox>
#include <QPrintDialog>
#include <QPrinter>
#include <QPushButton>
#include <QLayout>
#include <KLocalizedString>
#include "kplsettings.h"
#include "kpl.h"
#include "residualsdlg.h"
#include "kplscene.h"
#include "kplutils.h"
#include "kplview.h"

ResidualsDlg::ResidualsDlg(KplScene* sc, QWidget* parent) :
 QDialog(parent), scene(sc)
{
  setWindowTitle(i18n("Residuals"));
  QVBoxLayout* vbox = new QVBoxLayout(this);
  KplView* view  = new KplView(KplSettings::zoom());
  view->setScene(scene);
  view->setMinimumSize(view->sizeHint().boundedTo(0.8 *
    QApplication::desktop()->size()));
  vbox->addWidget(view);
  QDialogButtonBox* buttonBox = new QDialogButtonBox(QDialogButtonBox::Close);
  vbox->addWidget(buttonBox);
  connect(buttonBox->button(QDialogButtonBox::Close), SIGNAL(clicked()),
          SLOT(accept()));
  QPushButton* b;
  buttonBox->addButton(b = new QPushButton(i18n("Print")),
                       QDialogButtonBox::ActionRole);
  connect(b, SIGNAL(clicked()), SLOT(slotPrint()));
  buttonBox->addButton(b = new QPushButton(i18n("Save")),
                       QDialogButtonBox::ActionRole);
  connect(b, SIGNAL(clicked()), SLOT(slotSave()));
}

ResidualsDlg::~ResidualsDlg()
{
}

void ResidualsDlg::slotPrint()
{
  QPrinter printer(QPrinter::HighResolution);
  if (QPrintDialog(&printer).exec() == QDialog::Accepted)
    scene->print(&printer);
}

void ResidualsDlg::slotSave()
{
  QUrl url;
  if (KplUtils::getWriteUrl(this, url, KplSettings::currentDir(),
                            "*.plo (*.plo);;* (*)"))
    if (scene->savePlo(url))
      KplUtils::setCurrentDir(url);
}
