/***************************************************************************
                          rootdlg.h  -  description
                             -------------------
    begin                : Sam Jan 25 2003
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ROOTDLG_H
#define ROOTDLG_H

#include <QDialog>

class QComboBox;
class QDialogButtonBox;
class QPushButton;
class QTableWidget;
class KplDoc;
class DoubleEdit;
class SplineItem;

/**
  *@author Werner Stille
  */

/**
  * Root dialog class. Provides a dialog for finding the roots of a smoothing
  * cubic spline or its derivatives.
  * @author Werner Stille
  */
class RootDlg : public QDialog
{
   Q_OBJECT
public: 
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param spl pointer to spline item.
   */
  RootDlg(QWidget* parent, KplDoc* model, const SplineItem* spl);
  /** Destructor. */
  ~RootDlg();

protected:
  /**
   * Reimplemented to display help when key F1 is pressed.
   * @param e Key event.
   */
  virtual void keyPressEvent(QKeyEvent* e);
  QComboBox *eCond;
  QPushButton *update, *bExport;
  QTableWidget* table;
  QDialogButtonBox* buttonBox;
  KplDoc* m;
  DoubleEdit *eOff;
  const SplineItem *spl;

protected slots:
  /** Finds roots. */
  virtual void findRoots();
  /** Exports to file. */
  void slotExport();
  /** Displays help page. */
  void slotHelp();
};

#endif
