/***************************************************************************
                          settingsdlg.cpp  -  description
                             -------------------
    begin                : Fr Jun 4 2004
    copyright            : (C) 2017 by Werner Stille
    email                : kpl@werner-stille.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QButtonGroup>
#include <QCheckBox>
#include <QComboBox>
#include <QDesktopServices>
#include <QDoubleSpinBox>
#include <QFileInfo>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QPushButton>
#include <QRadioButton>
#include <QSpinBox>
#include <QThread>
#include <QTimer>
#include <QToolButton>
#include <KColorButton>
#include <KLocalizedString>
#include <KUrlRequester>
#include <KWindowConfig>
#include "brushdlg.h"
#include "choosefuncdlg.h"
#include "doubleedit.h"
#include "edittextdlg.h"
#include "errmoddlg.h"
#include "funcdlg.h"
#include "kpldoc.h"
#include "kplsettings.h"
#include "kplutils.h"
#include "settingsdlg.h"
#include "sourcewidget.h"
#include "symboldlg.h"
#include "utils.h"

EditFitParameterDlg::EditFitParameterDlg(QWidget* parent, double* p0,
                                         bool* bFit0, const QString& file,
                                         const QString& func) :
 QDialog(parent), bFit(bFit0), p(p0)
{
  setWindowTitle(i18n("Fit parameters"));
  QHBoxLayout* hbox = 0;
  QVBoxLayout *vbox0, *vbox;
  if (KplSettings::showSource()) {
    vbox0 = new QVBoxLayout(this);
    vbox0->addLayout(hbox = new QHBoxLayout);
    hbox->addLayout(vbox = new QVBoxLayout);
  } else
    vbox0 = vbox = new QVBoxLayout(this);
  QGroupBox* g = new QGroupBox;
  QGridLayout* grid = new QGridLayout;
  g->setLayout(grid);
  vbox->addWidget(g);
  char format = KplDoc::presentation();
  int prec = KplSettings::precision();
  int w = 0;
  for (int i = 0; i < FunItem::NumParMax; i++) {
    int ix = 3 * (i / 10);
    int iy = i % 10;
    grid->addWidget(enFit[i] = new QCheckBox(QString("p%1").arg(i)), iy, ix);
    enFit[i]->setChecked(bFit[i]);
    grid->addWidget(par[i] = new DoubleEdit(p[i], format, prec), iy,
                    ix + 1);
    if (!i)
      w = par[0]->fontMetrics().width(QString::number(-1e-123 / 3,
                                                      format, prec)) + 3;
    par[i]->setMinimumWidth(w);
  }
  for (int i = 0; i < ((FunItem::NumParMax - 1) / 10); i++)
    grid->addItem(new QSpacerItem(20, 10), 0, 3 * i + 2);
  if (KplSettings::showSource()) {
    QFileInfo fi(file);
    QString s = fi.absolutePath() + "/" + fi.baseName();
    QFile f(s + ".c");
    bool open = f.open(QIODevice::ReadOnly);
    if (!open) {
      f.setFileName(s + ".cpp");
      open = f.open(QIODevice::ReadOnly);
    }
    if (open) {
      SourceWidget *sw = new SourceWidget(&f, this);
      hbox->addWidget(sw, 1);
      sw->setMinimumWidth(400);
      sw->highlight(func);
    }
  }
  vbox0->addWidget(buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok |
                                                    QDialogButtonBox::Cancel));
  buttonBox->button(QDialogButtonBox::Ok)->setFocus();
  connect(buttonBox, SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
}

EditFitParameterDlg::~EditFitParameterDlg()
{
}

void EditFitParameterDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
    case QDialogButtonBox::Ok:
      for (int i = 0; i < FunItem::NumParMax; i++) {
        p[i] = par[i]->value();
        bFit[i] = enFit[i]->isChecked();
      }
      if (id == QDialogButtonBox::Ok)
        accept();
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}

static const double xSize[] = {84.1, 118.9, 59.7, 84.1, 42, 59.7, 29.7, 42,
                               21, 29.7, 14.8, 21, 21.59, 27.94};
static const double ySize[] = {118.9, 84.1, 84.1, 59.7, 59.7, 42, 42, 29.7,
                               29.7, 21, 21, 14.8, 27.94, 21.59};

SettingsDlg::SettingsDlg(QWidget* parent, const QString& name,
                         KConfigSkeleton* conf, KplDoc* m0) :
 KConfigDialog(parent, name, conf), m(m0)
{
  QWidget* page;
  QHBoxLayout* hbox2 = new QHBoxLayout(page = new QWidget);
  QVBoxLayout* vbox = new QVBoxLayout;
  hbox2->addLayout(vbox);
  QGroupBox* g = new QGroupBox(i18n("Page format "));
  vbox->addWidget(g);
  QVBoxLayout* vbox2 = new QVBoxLayout(g);
  vbox2->addWidget(format = new QComboBox);
  format->insertItems(0, QStringList() << i18n("Portrait A0")
                                       << i18n("Landscape A0")
                                       << i18n("Portrait A1")
                                       << i18n("Landscape A1")
                                       << i18n("Portrait A2")
                                       << i18n("Landscape A2")
                                       << i18n("Portrait A3")
                                       << i18n("Landscape A3")
                                       << i18n("Portrait A4")
                                       << i18n("Landscape A4")
                                       << i18n("Portrait A5")
                                       << i18n("Landscape A5")
                                       << i18n("Portrait Letter")
                                       << i18n("Landscape Letter")
                                       << i18n("Custom"));
  QHBoxLayout* hbox = new QHBoxLayout;
  vbox2->addLayout(hbox);
  QLabel* l;
  hbox->addWidget(l = new QLabel(i18n("Width")));
  hbox->addWidget(kcfg_FormatX = new QDoubleSpinBox);
  kcfg_FormatX->setMaximum(200);
  kcfg_FormatX->setSingleStep(0.1);
  kcfg_FormatX->setSuffix(" cm");
  kcfg_FormatX->setObjectName("kcfg_FormatX");
  l->setBuddy(kcfg_FormatX);
  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding));
  hbox->addWidget(l = new QLabel(i18n("Height")));
  hbox->addWidget(kcfg_FormatY = new QDoubleSpinBox);
  kcfg_FormatY->setMaximum(200);
  kcfg_FormatY->setSingleStep(0.1);
  kcfg_FormatY->setSuffix(" cm");
  kcfg_FormatY->setObjectName("kcfg_FormatY");
  l->setBuddy(kcfg_FormatY);
  int i;
  lFormat = sizeof(xSize) / sizeof(double);
  for (i = 0; i < lFormat; i++)
    if ((xSize[i] == KplSettings::formatX()) &&
        (ySize[i] == KplSettings::formatY())) {
      kcfg_FormatX->setEnabled(false);
      kcfg_FormatY->setEnabled(false);
      break;
    }
  format->setCurrentIndex(i);
  connect(format, SIGNAL(activated(int)), SLOT(slotFormatChanged(int)));
  vbox->addWidget(g = new QGroupBox(i18n("Default colors")));
  QGridLayout* grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("Frame")), 0, 0);
  grid->addWidget(kcfg_ColFrame = new KColorButton, 0, 1);
  kcfg_ColFrame->setObjectName("kcfg_ColFrame");
  l->setBuddy(kcfg_ColFrame);
  grid->addWidget(l = new QLabel(i18n("Grid")), 1, 0);
  grid->addWidget(kcfg_ColGrid = new KColorButton, 1, 1);
  kcfg_ColGrid->setObjectName("kcfg_ColGrid");
  l->setBuddy(kcfg_ColGrid);
  grid->addWidget(l = new QLabel(i18n("Data")), 2, 0);
  grid->addWidget(kcfg_ColData = new KColorButton, 2, 1);
  kcfg_ColData->setObjectName("kcfg_ColData");
  l->setBuddy(kcfg_ColData);
  vbox->addWidget(g = new QGroupBox(i18n("Input format")));
  hbox = new QHBoxLayout(g);
  hbox->addWidget(l = new QLabel(i18n("Decimal symbol")));
  hbox->addWidget(kcfg_DecimalCharacter = new QComboBox);
  kcfg_DecimalCharacter->setObjectName("kcfg_DecimalCharacter");
  kcfg_DecimalCharacter->insertItems(0, QStringList() << "." << ",");
  l->setBuddy(kcfg_DecimalCharacter);
  vbox->addWidget(g = new QGroupBox(i18n("Output format")));
  grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("Presentation")), 0, 0);
  grid->addWidget(kcfg_Presentation = new QComboBox, 0, 1);
  kcfg_Presentation->insertItems(0, QStringList() << "e" << "E" << "f" << "g"
                                                  << "G");
  kcfg_Presentation->setObjectName("kcfg_Presentation");
  l->setBuddy(kcfg_Presentation);
  grid->addWidget(l = new QLabel(i18n("Precision")), 1, 0);
  grid->addWidget(kcfg_Precision = new QSpinBox, 1, 1);
  kcfg_Precision->setObjectName("kcfg_Precision");
  l->setBuddy(kcfg_Precision);
  grid->addWidget(l = new QLabel(i18n("Separator")), 2, 0);
  grid->addWidget(kcfg_Separator = new QComboBox, 2, 1);
  kcfg_Separator->insertItems(0, QStringList() << i18n("Space") << i18n("Tab")
                                               << ",");
  kcfg_Separator->setObjectName("kcfg_Separator");
  l->setBuddy(kcfg_Separator);
  hbox2->addWidget(g = new QGroupBox(i18n("Options")));
  vbox2 = new QVBoxLayout(g);
  vbox2->addWidget(kcfg_ShowSource =
    new QCheckBox(i18n("Show function source")));
  kcfg_ShowSource->setObjectName("kcfg_ShowSource");
  vbox2->addWidget(kcfg_AutoFit = new QCheckBox(i18n("Autofit")));
  kcfg_AutoFit->setObjectName("kcfg_AutoFit");
  vbox2->addWidget(kcfg_AddItems = new QCheckBox(i18n("Add files")));
  kcfg_AddItems->setObjectName("kcfg_AddItems");
  vbox2->addWidget(kcfg_BoundingBox =
    new QCheckBox(i18n("Calculate bounding box")));
  kcfg_BoundingBox->setObjectName("kcfg_BoundingBox");
  vbox2->addWidget(kcfg_SaveAbsPath =
    new QCheckBox(i18n("Save absolute paths")));
  kcfg_SaveAbsPath->setObjectName("kcfg_SaveAbsPath");
  vbox2->addWidget(kcfg_Warning =
    new QCheckBox(i18n("Unsaved changes warning")));
  kcfg_Warning->setObjectName("kcfg_Warning");
  vbox2->addWidget(kcfg_Multithreading =
    new QCheckBox(i18n("Use multiple cores")));
  kcfg_Multithreading->setObjectName("kcfg_Multithreading");
  if (QThread::idealThreadCount() < 2)
    kcfg_Multithreading->setDisabled(true);
  addPage(page, i18n("Basic settings"), "preferences-system");
  vbox = new QVBoxLayout(page = new QWidget);
  vbox->addLayout(hbox = new QHBoxLayout);
  kcfg_GridMode = new QButtonGroup;
  hbox->addWidget(g = new QGroupBox(i18n("Axis mode")));
//   kcfg_GridMode->setObjectName("kcfg_GridMode");
  vbox2 = new QVBoxLayout(g);
  QRadioButton* rb;
  i = 0;
  vbox2->addWidget(rb = new QRadioButton(i18n("Frame only")));
  kcfg_GridMode->addButton(rb, i++);
  vbox2->addWidget(rb = new QRadioButton(i18n("Tics")));
  kcfg_GridMode->addButton(rb, i++);
  vbox2->addWidget(rb = new QRadioButton(i18n("Tics with numbers")));
  kcfg_GridMode->addButton(rb, i++);
  vbox2->addWidget(rb = new QRadioButton(i18n("Grid")));
  kcfg_GridMode->addButton(rb, i++);
  vbox2->addWidget(rb = new QRadioButton(i18n("Grid with numbers")));
  kcfg_GridMode->addButton(rb, i++);
  connect(kcfg_GridMode, SIGNAL(buttonClicked(int)),
          SLOT(slotGridModeChanged(int)));
  hbox->addLayout(vbox2 = new QVBoxLayout);
  vbox2->addWidget(g = new QGroupBox(i18n("Window")));
  grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("Left margin")), 0, 0);
  grid->addWidget(kcfg_X0 = new QDoubleSpinBox, 0, 1);
  kcfg_X0->setMaximum(200);
  kcfg_X0->setSingleStep(0.1);
  kcfg_X0->setSuffix(" cm");
  kcfg_X0->setObjectName("kcfg_X0");
  l->setBuddy(kcfg_X0);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid->addWidget(l = new QLabel(i18n("Width")), 0, 3);
  grid->addWidget(kcfg_Width = new QDoubleSpinBox, 0, 4);
  kcfg_Width->setRange(0.5, 200);
  kcfg_Width->setSingleStep(0.1);
  kcfg_Width->setSuffix(" cm");
  kcfg_Width->setObjectName("kcfg_Width");
  l->setBuddy(kcfg_Width);
  grid->addWidget(l = new QLabel(i18n("Bottom margin")), 1, 0);
  grid->addWidget(kcfg_Y0 = new QDoubleSpinBox, 1, 1);
  kcfg_Y0->setMaximum(200);
  kcfg_Y0->setSingleStep(0.1);
  kcfg_Y0->setSuffix(" cm");
  kcfg_Y0->setObjectName("kcfg_Y0");
  l->setBuddy(kcfg_Y0);
  grid->addWidget(l = new QLabel(i18n("Height")), 1, 3);
  grid->addWidget(kcfg_Height = new QDoubleSpinBox, 1, 4);
  kcfg_Height->setRange(0.5, 200);
  kcfg_Height->setSingleStep(0.1);
  kcfg_Height->setSuffix(" cm");
  kcfg_Height->setObjectName("kcfg_Height");
  l->setBuddy(kcfg_Height);
  vbox2->addWidget(g = new QGroupBox(i18n("Labels")));
  grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("x axis")), 0, 0);
  grid->addWidget(kcfg_TextX = new QLineEdit, 0, 1);
  kcfg_TextX->setObjectName("kcfg_TextX");
  l->setBuddy(kcfg_TextX);
  QToolButton* tb = new QToolButton;
  tb->setText("...");
  grid->addWidget(tb, 0, 2);
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetX()));
  grid->addWidget(l = new QLabel(i18n("y axis"), g), 1, 0);
  grid->addWidget(kcfg_TextY = new QLineEdit, 1, 1);
  kcfg_TextY->setObjectName("kcfg_TextY");
  l->setBuddy(kcfg_TextY);
  grid->addWidget(tb = new QToolButton, 1, 2);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetY()));
  grid->addWidget(l = new QLabel(i18n("Heading"), g), 2, 0);
  grid->addLayout(hbox = new QHBoxLayout, 2, 1);
  hbox->addWidget(kcfg_AutoPathHeader = new QCheckBox(i18n("= path")));
  kcfg_AutoPathHeader->setObjectName("kcfg_AutoPathHeader");
  hbox->addWidget(kcfg_TextH = new QLineEdit);
  kcfg_TextH->setObjectName("kcfg_TextH");
  l->setBuddy(kcfg_TextH);
  grid->addWidget(editH = new QToolButton, 2, 2);
  editH->setText("...");
  connect(editH, SIGNAL(clicked()), SLOT(slotEditLetH()));
  vbox->addLayout(hbox = new QHBoxLayout);
  hbox->addWidget(g = new QGroupBox(i18n("Data")));
  grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("x column")), 0, 0);
  grid->addWidget(kcfg_IX = new QSpinBox, 0, 1);
  kcfg_IX->setObjectName("kcfg_IX");
  l->setBuddy(kcfg_IX);
  grid->addWidget(kcfg_LogX = new QCheckBox("log"), 0, 2);
  kcfg_LogX->setObjectName("kcfg_LogX");
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 3);
  grid->addWidget(l = new QLabel(i18n("y column"), g), 1, 0);
  grid->addWidget(kcfg_IY = new QSpinBox, 1, 1);
  kcfg_IY->setObjectName("kcfg_IY");
  l->setBuddy(kcfg_IY);
  grid->addWidget(kcfg_LogY = new QCheckBox("log"), 1, 2);
  kcfg_LogY->setObjectName("kcfg_LogY");
  grid->addWidget(l = new QLabel(i18n("Error column")), 2, 0);
  grid->addWidget(kcfg_IE = new QSpinBox, 2, 1);
  kcfg_IE->setObjectName("kcfg_IE");
  l->setBuddy(kcfg_IE);
  kcfg_IE->setEnabled(KplSettings::errBars());
  grid->addWidget(kcfg_ErrBars = new QCheckBox(i18n("Error bar")), 2, 2);
  kcfg_ErrBars->setObjectName("kcfg_ErrBars");
  grid->addWidget(kcfg_Normalization = new QCheckBox(i18n("Normalization")), 3,
                  0);
  kcfg_Normalization->setObjectName("kcfg_Normalization");
  connect(kcfg_AutoPathHeader, SIGNAL(toggled(bool)),
          SLOT(disableHeading(bool)));
  connect(kcfg_LogX, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(kcfg_LogY, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(kcfg_ErrBars, SIGNAL(toggled(bool)), kcfg_IE,
          SLOT(setEnabled(bool)));
  connect(kcfg_Normalization, SIGNAL(toggled(bool)),
          SLOT(slotNormToggled(bool)));
  hbox->addWidget(g = new QGroupBox(i18n("Representation")));
  grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("Symbol")), 0, 0);
  grid->addWidget(kcfg_Symbol = new QSpinBox, 0, 1);
  kcfg_Symbol->setRange(-17, 20);
  kcfg_Symbol->setObjectName("kcfg_Symbol");
  l->setBuddy(kcfg_Symbol);
  grid->addWidget(bSymb = new SymbolButton(kcfg_Symbol), 0, 2);
  grid->addWidget(l = new QLabel(i18n("Symbol size")), 1, 0);
  grid->addWidget(kcfg_RelSize = new QSpinBox, 1, 1);
  kcfg_RelSize->setRange(10, 800);
  kcfg_RelSize->setSuffix(" %");
  kcfg_RelSize->setObjectName("kcfg_RelSize");
  l->setBuddy(kcfg_RelSize);
  grid->addWidget(l = new QLabel(i18n("Fill pattern")), 2, 0);
  grid->addWidget(kcfg_FillStyle = new BrushButton, 2, 1);
  kcfg_FillStyle->setObjectName("kcfg_FillStyle");
  l->setBuddy(kcfg_FillStyle);
  connect(kcfg_FillStyle, SIGNAL(changed(int)), SLOT(slotFillChanged(int)));
  slotLogToggled(KplSettings::logX() || KplSettings::logY());
  disableHeading(KplSettings::autoPathHeader());
  addPage(page, i18n("Autoplot settings"), "kpl");
  vbox = new QVBoxLayout(page = new QWidget);
  g = new QGroupBox("y(x)");
  vbox->addWidget(g);
  grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("Library")), 0, 0);
  grid->addWidget(kcfg_FitPath = new KUrlRequester, 0, 1);
  kcfg_FitPath->setObjectName("kcfg_FitPath");
  kcfg_FitPath->setStartDir(Utils::url(KplSettings::currentDir()));
  kcfg_FitPath->setFilter("*.so\n*");
  kcfg_FitPath->setSizePolicy(QSizePolicy::MinimumExpanding,
                              QSizePolicy::Fixed);
  l->setBuddy(kcfg_FitPath);
  grid->addWidget(l = new QLabel(i18n("Function")), 1, 0);
  grid->addLayout(hbox = new QHBoxLayout, 1, 1);
  hbox->addWidget(kcfg_FitName = new QLineEdit);
  kcfg_FitName->setObjectName("kcfg_FitName");
  l->setBuddy(kcfg_FitName);
  QPushButton* b2 = new QPushButton("...");
  b2->setFixedSize(kcfg_FitPath->button()->size());
  hbox->addWidget(b2);
  connect(b2, SIGNAL(clicked()), SLOT(slotFunc()));
  QPushButton* b;
  grid->addWidget(new QLabel(i18n("Parameter")), 2, 0);
  grid->addLayout(hbox = new QHBoxLayout, 2, 1);
  hbox->addWidget(b = new QPushButton(i18n("Edit")));
  connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Load")));
  connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Save")));
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
  vbox->addLayout(hbox = new QHBoxLayout);
  hbox->addWidget(g = new QGroupBox(i18n("Parameter fit")));
  grid = new QGridLayout(g);
  grid->addWidget(kcfg_FitNonLinear = new QCheckBox(i18n("Nonlinear fit")), 0,
                  0, 1, 2);
  kcfg_FitNonLinear->setObjectName("kcfg_FitNonLinear");
  grid->addWidget(kcfg_FitErrorColumn = new QCheckBox(i18n("Error column")), 1,
                  0, 1, 2);
  kcfg_FitErrorColumn->setObjectName("kcfg_FitErrorColumn");
  grid->addWidget(errMod = new QPushButton(i18n("Model"), g), 1, 4);
  connect(errMod, SIGNAL(clicked()), SLOT(slotErrMod()));
  grid->addWidget(l = new QLabel(i18n("Maximum iterations")), 2, 0);
  grid->addWidget(kcfg_FitMaxIterations = new QSpinBox, 2, 1);
  kcfg_FitMaxIterations->setRange(1, 200);
  kcfg_FitMaxIterations->setObjectName("kcfg_FitMaxIterations");
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 2, 2);
  l->setBuddy(kcfg_FitMaxIterations);
  grid->addWidget(l = new QLabel(i18n("Tolerance")), 2, 3);
  grid->addWidget(kcfg_FitTolerance = new DoubleEdit(0, 0, 1, 'g', 6),
                  2, 4);
  kcfg_FitTolerance->setObjectName("kcfg_FitTolerance");
  l->setBuddy(kcfg_FitTolerance);
  grid->addWidget(kcfg_FitShowDialog = new QCheckBox(i18n("Show dialog")), 3,
                  0, 1, 2);
  kcfg_FitShowDialog->setObjectName("kcfg_FitShowDialog");
  grid->addWidget(kcfg_FitSaveParam = new QCheckBox(i18n("Save parameters")),
                  4, 0, 1, 2);
  kcfg_FitSaveParam->setObjectName("kcfg_FitSaveParam");
  grid->addWidget(kcfg_FitFollow = new QCheckBox(i18n("Follow parameters")), 5,
                  0, 1, 2);
  kcfg_FitFollow->setObjectName("kcfg_FitFollow");
  hbox->addWidget(g = new QGroupBox(i18n("Representation")));
  grid = new QGridLayout(g);
  grid->addWidget(l = new QLabel(i18n("Symbol")), 0, 0);
  grid->addWidget(kcfg_FitSymbol = new QSpinBox, 0, 1);
  kcfg_FitSymbol->setRange(-17, 9);
  kcfg_FitSymbol->setObjectName("kcfg_FitSymbol");
  l->setBuddy(kcfg_FitSymbol);
  grid->addWidget(bSymbFit = new SymbolButton(kcfg_FitSymbol), 0, 2);
  grid->addWidget(l = new QLabel(i18n("Color")), 1, 0);
  grid->addWidget(kcfg_FitColor = new KColorButton, 1, 1);
  kcfg_FitColor->setObjectName("kcfg_FitColor");
  l->setBuddy(kcfg_FitColor);
  grid->addWidget(l = new QLabel(i18n("Fill pattern")), 2, 0);
  grid->addWidget(kcfg_FitFillStyle = new BrushButton, 2, 1);
  kcfg_FitFillStyle->setObjectName("kcfg_FitFillStyle");
  l->setBuddy(kcfg_FitFillStyle);
  connect(kcfg_FitFillStyle, SIGNAL(changed(int)),
          SLOT(slotFillFitChanged(int)));
  addPage(page, i18n("Autofit settings"), "tools-wizard");
  resize(minimumSize());
  connect(kcfg_FitErrorColumn, SIGNAL(toggled(bool)),
          SLOT(disableErrMod(bool)));
  connect(kcfg_FitNonLinear, SIGNAL(toggled(bool)), SLOT(enableNonLin(bool)));
  disableErrMod(KplSettings::fitErrorColumn());
  enableNonLin(KplSettings::fitNonLinear());
  bool style = KplSettings::fillStyle();
  kcfg_Symbol->setDisabled(style);
  bSymb->setDisabled(style);
  kcfg_RelSize->setDisabled(style);
  style = KplSettings::fitFillStyle();
  kcfg_FitSymbol->setDisabled(style);
  bSymbFit->setDisabled(style);
  QTimer::singleShot(0, this, SLOT(slotRestoreSize()));
}

SettingsDlg::~SettingsDlg()
{
  KConfigGroup config = KSharedConfig::openConfig()->group("SettingsDlg");
  KWindowConfig::saveWindowSize(windowHandle(), config);
}

bool SettingsDlg::fitParChanged()
{
  for (int i = 0; i < FunItem::NumParMax; ++i)
    if (fitPar[i] != fitPar0[i])
      return true;
  return false;
}

bool SettingsDlg::hasChanged()
{
  return ((gridMode != gridMode0) || fitParChanged() ||
          FunItem::parChanged(pFit, pFit0) ||
          FunItem::errorStructChanged(err, err0));
}

bool SettingsDlg::isDefault()
{
  bool useDef = KplSettings::self()->useDefaults(true);
  FunItem::DataErrorStruct errDef = m->errorModel();
  double pFitDef[FunItem::NumParMax];
  FunItem::stringToPar(KplSettings::fitParam(), pFitDef);
  bool fitParDef[FunItem::NumParMax];
  KplDoc::readFitPar(fitParDef);
  int gridModeDef = KplSettings::gridMode();
  KplSettings::self()->useDefaults(useDef);
  if (gridMode != gridModeDef)
    return false;
  if (FunItem::errorStructChanged(err, errDef))
    return false;
  for (int i = 0; i < FunItem::NumParMax; ++i)
    if ((pFit[i] != pFitDef[i]) || (fitPar[i] != fitParDef[i]))
      return false;
  return true;
}

void SettingsDlg::useDefaults(bool defaults)
{
  KplSettings::self()->useDefaults(defaults);
  err = m->errorModel();
  FunItem::stringToPar(KplSettings::fitParam(), pFit);
  KplDoc::readFitPar(fitPar);
  gridMode = KplSettings::gridMode();
  KplSettings::self()->useDefaults(false);
}

void SettingsDlg::disableErrMod(bool off)
{
  errMod->setEnabled(!off);
}

void SettingsDlg::disableHeading(bool off)
{
  kcfg_TextH->setEnabled(!off);
}

void SettingsDlg::enableNonLin(bool on)
{
  kcfg_FitTolerance->setEnabled(on);
  kcfg_FitMaxIterations->setEnabled(on);
}

void SettingsDlg::showHelp()
{
  QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/settings.html")));
}

void SettingsDlg::slotEditLetX()
{
  EditTextDlg::editText(this, kcfg_TextX);
}

void SettingsDlg::slotEditLetY()
{
  EditTextDlg::editText(this, kcfg_TextY);
}

void SettingsDlg::slotEditLetH()
{
  EditTextDlg::editText(this, kcfg_TextH);
}

void SettingsDlg::slotEditParameter()
{
  EditFitParameterDlg dlg(this, pFit, fitPar, kcfg_FitPath->url().path(),
                          kcfg_FitName->text());
  if (dlg.exec())
    if (fitParChanged() || FunItem::parChanged(pFit, pFit0))
      updateButtons();
}

void SettingsDlg::slotErrMod()
{
  ErrModDlg dlg(this, m, &err);
  if (dlg.exec())
    if (FunItem::errorStructChanged(err, err0))
      updateButtons();
}

void SettingsDlg::slotFillChanged(int brush)
{
  kcfg_Symbol->setDisabled(brush);
  bSymb->setDisabled(brush);
  kcfg_RelSize->setDisabled(brush);
}

void SettingsDlg::slotFillFitChanged(int brush)
{
  kcfg_FitSymbol->setDisabled(brush);
  bSymbFit->setDisabled(brush);
}

void SettingsDlg::slotFormatChanged(int i)
{
  bool standard = (i < lFormat);
  kcfg_FormatX->setDisabled(standard);
  kcfg_FormatY->setDisabled(standard);
  if (standard) {
    kcfg_FormatX->setValue(xSize[i]);
    kcfg_FormatY->setValue(ySize[i]);
  }
}

void SettingsDlg::slotGridModeChanged(int i)
{
  gridMode = i;
  if (gridMode != gridMode0)
    updateButtons();
}

void SettingsDlg::slotFunc()
{
  KplUtils::getFunc(this, kcfg_FitPath->url(), kcfg_FitName);
}

void SettingsDlg::slotGetParameter()
{
  if (m->getPar(this, pFit))
    if (FunItem::parChanged(pFit, pFit0))
      updateButtons();
}

void SettingsDlg::slotLogToggled(bool state)
{
  if (state)
    kcfg_Normalization->setChecked(false);
}

void SettingsDlg::slotNormToggled(bool state)
{
  if (state) {
    kcfg_LogX->setChecked(false);
    kcfg_LogY->setChecked(false);
  }
}

void SettingsDlg::slotRestoreSize()
{
  KWindowConfig::restoreWindowSize(windowHandle(),
    KSharedConfig::openConfig()->group("SettingsDlg"));
}

void SettingsDlg::slotSaveParameter()
{
  m->saveFunPar(this, pFit);
}

void SettingsDlg::updateSettings()
{
  bool bGridMode = (gridMode != gridMode0);
  bool bErr = FunItem::errorStructChanged(err, err0);
  bool bFit = fitParChanged();
  bool bPar = FunItem::parChanged(pFit, pFit0);
  if (bGridMode) {
    KplSettings::setGridMode(gridMode);
    gridMode0 = gridMode;
  }
  if (bErr) {
    if (err.fitErrCol != err0.fitErrCol)
      KplSettings::setFitErrorColumn(err.fitErrCol);
    if (err.errModPath != err0.errModPath)
      KplSettings::setErrorModelPath(err.errModPath);
    if (err.errModName != err0.errModName)
      KplSettings::setErrorModelName(err.errModName);
    if (err.errModArg != err0.errModArg)
      KplSettings::setErrorModelArgument(err.errModArg);
    if (FunItem::parChanged(err.parErrMod, err0.parErrMod))
      KplSettings::setErrorModelParam(
        FunItem::parToString(err.parErrMod, m->presentation(),
                             KplSettings::precision()));
    err0 = err;
    m->setErrorModel(err);
  }
  if (bFit) {
    QStringList list;
    for (int i = 0; i < FunItem::NumParMax; ++i) {
      m->fitPar()[i] = fitPar0[i] = fitPar[i];
      list.append(fitPar[i] ? "1" : "0");
    }
    KplSettings::setFitEnableParam(list.join(" "));
  }
  if (bPar) {
    for (int i = 0; i < FunItem::NumParMax; ++i)
      m->pFit()[i] = pFit0[i] = pFit[i];
    KplSettings::setFitParam(FunItem::parToString(pFit, m->presentation(),
                                                  KplSettings::precision()));
  }
  if (bGridMode || bErr || bFit || bPar)
    KplSettings::self()->save();
}

void SettingsDlg::updateWidgets()
{
  useDefaults(false);
  err0 = err;
  for (int i = 0; i < FunItem::NumParMax; ++i) {
    pFit0[i] = pFit[i];
    fitPar0[i] = fitPar[i];
  }
  gridMode0 = gridMode;
  kcfg_GridMode->button(gridMode)->setChecked(true);
}

void SettingsDlg::updateWidgetsDefault()
{
  useDefaults(true);
  kcfg_GridMode->button(gridMode)->setChecked(true);
}
