//
// C++ Implementation: threeddlg
//
// Description: 
//
//
// Author: Werner Stille <kpl@werner-stille.de>, (C) 2017
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <QtMath>
#include <QButtonGroup>
#include <QCheckBox>
#include <QDesktopServices>
#include <QDoubleSpinBox>
#include <QFile>
#include <QGroupBox>
#include <QKeyEvent>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QRadioButton>
#include <QTimer>
#include <QToolButton>
#include <KColorButton>
#include <KLocalizedString>
#include <KMessageBox>
#include <KUrlRequester>
#include <KWindowConfig>
#include "array3ditem.h"
#include "doubleedit.h"
#include "editdlg.h"
#include "editparameterdlg.h"
#include "edittextdlg.h"
#include "filedialog.h"
#include "fun3ditem.h"
#include "funcdlg.h"
#include "kpl.h"
#include "kpldoc.h"
#include "kplsettings.h"
#include "kplutils.h"
#include "spinbox.h"
#include "threeddlg.h"
#include "utils.h"

ThreeDDlg::ThreeDDlg(QWidget* parent, KplDoc* model, ThreeDItem* td0,
                     const QPoint& pos, const QString& caption,
                     const QString& name1) :
 KPageDialog(parent), m(model), td(td0)
{
  setFaceType(KPageDialog::Tabbed);
  setWindowTitle(caption);
  setStandardButtons(QDialogButtonBox::Help | QDialogButtonBox::Ok |
                     QDialogButtonBox::Apply | QDialogButtonBox::Cancel);
  bExport = new QPushButton(i18n("&Export"));
  buttonBox()->addButton(bExport, QDialogButtonBox::ActionRole);
  page1 = new QWidget;
  vbox1 = new QVBoxLayout(page1);
  g1 = new QGroupBox(i18n("Range"));
  addPage(page1, name1);
  QWidget* page;
  QGridLayout* grid = new QGridLayout(page = new QWidget);
  QGroupBox* g;
  grid->addWidget(g = new QGroupBox(i18n("Window")), 0, 0, 1, 2);
  QGridLayout* grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel(i18n("Left margin")), 0, 0);
  grid2->addWidget(ex0 = new QDoubleSpinBox, 0, 1);
  ex0->setMaximum(200);
  ex0->setSingleStep(0.1);
  ex0->setSuffix(" cm");
  ex0->setValue(td->x0());
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid2->addWidget(new QLabel(i18n("Width")), 0, 3);
  grid2->addWidget(exl = new QDoubleSpinBox, 0, 4);
  exl->setRange(0.5, 200);
  exl->setSingleStep(0.1);
  exl->setSuffix(" cm");
  exl->setValue(td->width());
  grid2->addWidget(new QLabel(i18n("Bottom margin")), 1, 0);
  grid2->addWidget(ey0 = new QDoubleSpinBox, 1, 1);
  ey0->setMaximum(200);
  ey0->setSingleStep(0.1);
  ey0->setSuffix(" cm");
  ey0->setValue(td->y0());
  grid2->addWidget(new QLabel(i18n("Height")), 1, 3);
  grid2->addWidget(eyl = new QDoubleSpinBox, 1, 4);
  eyl->setRange(0.5, 200);
  eyl->setSingleStep(0.1);
  eyl->setSuffix(" cm");
  eyl->setValue(td->height());
  grid->addWidget(g = new QGroupBox(i18n("Reference point")), 0, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("x"), 0, 0);
  grid2->addWidget(eXref = new QDoubleSpinBox, 0, 1);
  eXref->setMaximum(200);
  eXref->setSingleStep(0.1);
  eXref->setSuffix(" cm");
  eXref->setValue(td->xRef());
  grid2->addWidget(new QLabel("y"), 1, 0);
  grid2->addWidget(eYref = new QDoubleSpinBox, 1, 1);
  eYref->setMaximum(200);
  eYref->setSingleStep(0.1);
  eYref->setSuffix(" cm");
  eYref->setValue(td->yRef());
  grid->addWidget(g = new QGroupBox(i18n("3D box")), 1, 0);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("x"), 0, 0);
  grid2->addWidget(eBoxx = new QDoubleSpinBox, 0, 1);
  eBoxx->setRange(0.5, 200);
  eBoxx->setSingleStep(0.1);
  eBoxx->setSuffix(" cm");
  eBoxx->setValue(td->boxX());
  grid2->addWidget(new QLabel("y"), 1, 0);
  grid2->addWidget(eBoxy = new QDoubleSpinBox, 1, 1);
  eBoxy->setRange(0.5, 200);
  eBoxy->setSingleStep(0.1);
  eBoxy->setSuffix(" cm");
  eBoxy->setValue(td->boxY());
  grid2->addWidget(new QLabel("z"), 2, 0);
  grid2->addWidget(eBoxz = new QDoubleSpinBox, 2, 1);
  eBoxz->setRange(0.5, 200);
  eBoxz->setSingleStep(0.1);
  eBoxz->setSuffix(" cm");
  eBoxz->setValue(td->boxZ());
  grid->addWidget(g = new QGroupBox(i18n("Rotation")), 1, 1);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("phi"), 0, 0);
  grid2->addWidget(ePhi = new QDoubleSpinBox, 0, 1);
  ePhi->setRange(-360, 360);
  ePhi->setSuffix(QString::fromUtf8("\302\260"));
  ePhi->setValue(td->phi());
  grid2->addWidget(new QLabel("theta"), 1, 0);
  grid2->addWidget(eTheta = new QDoubleSpinBox, 1, 1);
  eTheta->setRange(-360, 360);
  eTheta->setSuffix(QString::fromUtf8("\302\260"));
  eTheta->setValue(td->theta());
  grid->addWidget(g = new QGroupBox(i18n("Automatic")), 1, 2);
  QVBoxLayout* vbox2 = new QVBoxLayout(g);
  vbox2->addWidget(autoNorm = new QCheckBox(i18n("Normalization")));
  autoNorm->setChecked(KplSettings::normalization() &&
                       (!(td->logX() || td->logY() || td->logZ())));
  bAuto = new QPushButton(i18n("Autoscale"));
  vbox2->addWidget(bAuto);
  grid->addWidget(g = new QGroupBox(i18n("Axes")), 2, 0, 1, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("xtic"), 0, 0);
  grid2->addWidget(exTic = new DoubleEdit(td->xTic()), 0, 1);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid2->addWidget(new QLabel("mticx"), 0, 3);
  grid2->addWidget(emTicx = new SpinBox(td->minTicX(), 0, 10), 0, 4);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 5);
  grid2->addWidget(new QLabel("ndigx"), 0, 6);
  grid2->addWidget(enDigx = new SpinBox(td->numDigX(), -1, 8), 0, 7);
  enDigx->setEnabled(!td->logX());
  grid2->addWidget(new QLabel("ytic"), 1, 0);
  grid2->addWidget(eyTic = new DoubleEdit(td->yTic()), 1, 1);
  grid2->addWidget(new QLabel("mticy"), 1, 3);
  grid2->addWidget(emTicy = new SpinBox(td->minTicY(), 0, 10), 1, 4);
  grid2->addWidget(new QLabel("ndigy"), 1, 6);
  grid2->addWidget(enDigy = new SpinBox(td->numDigY(), -1, 8), 1, 7);
  enDigy->setEnabled(!td->logY());
  grid2->addWidget(new QLabel("ztic"), 2, 0);
  grid2->addWidget(ezTic = new DoubleEdit(td->zTic()), 2, 1);
  grid2->addWidget(new QLabel("mticz"), 2, 3);
  grid2->addWidget(emTicz = new SpinBox(td->minTicZ(), 0, 10, 1), 2, 4);
  grid2->addWidget(new QLabel("ndigz"), 2, 6);
  grid2->addWidget(enDigz = new SpinBox(td->numDigZ(), -1, 8), 2, 7);
  enDigz->setEnabled(!td->logZ());
  grid->addWidget(g = new QGroupBox(i18n("Normalization")), 2, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("x"), 0, 0);
  grid2->addWidget(efx = new DoubleEdit(td->fX()), 0, 1);
  grid2->addWidget(new QLabel("y"), 1, 0);
  grid2->addWidget(efy = new DoubleEdit(td->fY()), 1, 1);
  grid2->addWidget(new QLabel("z"), 2, 0);
  grid2->addWidget(efz = new DoubleEdit(td->fZ()), 2, 1);
  addPage(page, i18n("Scaling"));
  grid = new QGridLayout(page = new QWidget);
  bg1 = new QButtonGroup;
  grid->addWidget(g = new QGroupBox, 0, 0);
  g->setTitle(i18n("Axis mode"));
  QVBoxLayout* vbox = new QVBoxLayout(g);
  QRadioButton* rb;
  int i = 0;
  vbox->addWidget(rb = new QRadioButton(i18n("No Tics")));
  bg1->addButton(rb, i++);
  vbox->addWidget(rb = new QRadioButton(i18n("Tics")));
  bg1->addButton(rb, i++);
  vbox->addWidget(rb = new QRadioButton(i18n("Tics with numbers")));
  bg1->addButton(rb, i);
  bg1->button(td->gridMode())->setChecked(true);
  bg2 = new QButtonGroup;
  grid->addWidget(g = new QGroupBox, 0, 1);
  g->setTitle(i18n("3D mode"));
  vbox = new QVBoxLayout(g);
  i = 0;
  vbox->addWidget(rb = new QRadioButton("x"));
  bg2->addButton(rb, i++);
  vbox->addWidget(rb = new QRadioButton("z"));
  bg2->addButton(rb, i++);
  vbox->addWidget(rb = new QRadioButton(i18n("x and z")));
  bg2->addButton(rb, i);
  bg2->button(td->mode3D())->setChecked(true);
  grid->addWidget(g = new QGroupBox(i18n("Colors")), 1, 0);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel(i18n("Frame")), 0, 0);
  grid2->addWidget(colFrame = new KColorButton(QColor(td->colorFrame())), 0,
                   1);
  grid2->addWidget(new QLabel(i18n("Tics")), 1, 0);
  grid2->addWidget(colGrid = new KColorButton(QColor(td->colorGrid())), 1, 1);
  grid2->addWidget(new QLabel(i18n("Data")), 2, 0);
  grid2->addWidget(colData = new KColorButton(QColor(td->colorData())), 2, 1);
  grid->addWidget(g = new QGroupBox(i18n("General")), 1, 1);
  vbox = new QVBoxLayout(g);
  vbox->addWidget(modeFrame = new QCheckBox(i18n("Show frame")));
  modeFrame->setChecked(td->showFrame());
  QHBoxLayout* hbox = new QHBoxLayout;
  vbox->addLayout(hbox);
  hbox->addWidget(new QLabel(i18n("Line width")));
  hbox->addWidget(eRelSiz = new SpinBox(100 * td->relSize(), 10, 800));
  eRelSiz->setSuffix(" %");
  grid->addWidget(g = new QGroupBox(i18n("Labels")), 2, 0, 1, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("iex"), 0, 0);
  grid2->addWidget(ex = new SpinBox(td->expX(), -300, 300), 0, 1);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::Minimum), 0, 2);
  grid2->addWidget(new QLabel(i18n("x axis")), 0, 3);
  grid2->addWidget(eLetX = new QLineEdit(td->textX()), 0, 4);
  QToolButton* tb;
  grid2->addWidget(tb = new QToolButton, 0, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetX()));
  grid2->addWidget(new QLabel("iey"), 1, 0);
  grid2->addWidget(ey = new SpinBox(td->expY(), -300, 300), 1, 1);
  grid2->addWidget(new QLabel(i18n("y axis")), 1, 3);
  grid2->addWidget(eLetY = new QLineEdit(td->textY()), 1, 4);
  grid2->addWidget(tb = new QToolButton, 1, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetY()));
  grid2->addWidget(new QLabel("iez"), 2, 0);
  grid2->addWidget(ez = new SpinBox(td->expZ(), -300, 300), 2, 1);
  grid2->addWidget(new QLabel(i18n("z axis")), 2, 3);
  grid2->addWidget(eLetZ = new QLineEdit(td->textZ()), 2, 4);
  grid2->addWidget(tb = new QToolButton, 2, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetZ()));
  grid2->addWidget(new QLabel(i18n("Heading")), 3, 3);
  grid2->addWidget(eLetH = new QLineEdit(td->header()), 3, 4);
  grid2->addWidget(tb = new QToolButton, 3, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetH()));
  connect(autoNorm, SIGNAL(toggled(bool)), SLOT(slotNormToggled(bool)));
  addPage(page, i18n("Representation"));
  move(pos);
  buttonBox()->button(QDialogButtonBox::Ok)->setEnabled(false);
  buttonBox()->button(QDialogButtonBox::Apply)->setEnabled(false);
}

ThreeDDlg::~ThreeDDlg()
{
}

void ThreeDDlg::autoScaleXZ(double xmin, double xmax, double zmin, double zmax)
{
  int iex, iez;
  double fx, fz;
  if (autoNorm->isChecked()) {
    ScaledItem::exponent(qMax(fabs(xmin), fabs(xmax)), iex, fx);
    ScaledItem::exponent(qMax(fabs(zmin), fabs(zmax)), iez, fz);
  } else {
    iex = iez = 0;
    fx = fz = 1;
  }
  efx->setValue(fx);
  ex->setValue(iex);
  efz->setValue(fz);
  ez->setValue(iez);
  int mtic, ndig;
  double vmin, vmax, vtic;
  FrameItem::autoScale(vmin, vmax, vtic, mtic, ndig, xmin, xmax, fx, 0.25,
                       logX->isChecked());
  exMin->setValue(vmin / fx);
  exMax->setValue(vmax / fx);
  exTic->setValue(vtic);
  emTicx->setValue(mtic);
  enDigx->setValue(ndig);
  FrameItem::autoScale(vmin, vmax, vtic, mtic, ndig, zmin, zmax, fz, 0.25,
                       logZ->isChecked());
  ezMin->setValue(vmin / fz);
  ezMax->setValue(vmax / fz);
  ezTic->setValue(vtic);
  emTicz->setValue(mtic);
  enDigz->setValue(ndig);
}

void ThreeDDlg::autoScaleY(double ymin, double ymax)
{
  double w = exl->value();
  double h = eyl->value();
  double r = h / w;
  double phi = qDegreesToRadians(ePhi->value());
  double theta = qDegreesToRadians(eTheta->value());
  double ctheta = cos(theta);
  double xr = fabs(sin(phi)) + fabs(cos(phi));
  double b = 0.7 * qMin(w / xr, h / (r * fabs(ctheta) +
                                     xr * fabs(sin(theta))));
  eBoxx->setValue(b);
  eBoxz->setValue(b);
  eBoxy->setValue(b * r);
  eXref->setValue(0.5 * w);
  eYref->setValue(0.5 * (h - b * r * ctheta));
  int iey;
  double fy;
  if (autoNorm->isChecked())
    ScaledItem::exponent(qMax(fabs(ymin), fabs(ymax)), iey, fy);
  else {
    iey = 0;
    fy = 1;
  }
  efy->setValue(fy);
  ey->setValue(iey);
  int mtic, ndig;
  double vmin, vmax, vtic;
  FrameItem::autoScale(vmin, vmax, vtic, mtic, ndig, ymin, ymax, fy, 0.25,
                       logY->isChecked());
  eyMin->setValue(vmin / fy);
  eyMax->setValue(vmax / fy);
  eyTic->setValue(vtic);
  emTicy->setValue(mtic);
  enDigy->setValue(ndig);
}

void ThreeDDlg::getValues(ThreeDItem* ti)
{
  ti->setXMin(exMin->value());
  ti->setXMax(exMax->value());
  ti->setDeltaX(exStep->value());
  ti->setLogX(logX->isChecked());
  ti->setYMin(eyMin->value());
  ti->setYMax(eyMax->value());
  ti->setLogY(logY->isChecked());
  ti->setZMin(ezMin->value());
  ti->setZMax(ezMax->value());
  ti->setDeltaZ(ezStep->value());
  ti->setLogZ(logZ->isChecked());
  ti->setX0(ex0->value());
  ti->setWidth(exl->value());
  ti->setY0(ey0->value());
  ti->setHeight(eyl->value());
  ti->setFX(efx->value());
  ti->setFY(efy->value());
  ti->setFZ(efz->value());
  ti->setBoxX(eBoxx->value());
  ti->setBoxY(eBoxy->value());
  ti->setBoxZ(eBoxz->value());
  ti->setXTic(exTic->value());
  ti->setYTic(eyTic->value());
  ti->setZTic(ezTic->value());
  ti->setMinTicX(emTicx->value());
  ti->setMinTicY(emTicy->value());
  ti->setMinTicZ(emTicz->value());
  ti->setNumDigX(enDigx->value());
  ti->setNumDigY(enDigy->value());
  ti->setNumDigZ(enDigz->value());
  ti->setPhi(ePhi->value());
  ti->setTheta(eTheta->value());
  ti->setXRef(eXref->value());
  ti->setYRef(eYref->value());
  ti->setGridMode(FrameItem::GridMode(bg1->checkedId()));
  ti->setMode3D(ThreeDItem::Mode3D(bg2->checkedId()));
  ti->setShowFrame(modeFrame->isChecked());
  ti->setRelSize(0.01 * eRelSiz->value());
  ti->setColorFrame(colFrame->color().rgb());
  ti->setColorGrid(colGrid->color().rgb());
  ti->setColorData(colData->color().rgb());
  ti->setExpX(ex->value());
  ti->setExpY(ey->value());
  ti->setExpZ(ez->value());
  ti->setTextX(eLetX->text());
  ti->setTextY(eLetY->text());
  ti->setTextZ(eLetZ->text());
  ti->setHeader(eLetH->text());
}

void ThreeDDlg::initRange(QGridLayout* grid, int row)
{
  g1->setLayout(grid);
  char format = KplDoc::presentation();
  int prec = KplSettings::precision();
  grid->addWidget(new QLabel("xmin"), row, 0);
  grid->addWidget(exMin = new DoubleEdit(td->xMin(), format, prec), row, 1);
  int w = exMin->fontMetrics().width(QString::number(-1e-123 / 3, format,
                                                     prec)) + 3;
  exMin->setMinimumWidth(w);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row,
                                2);
  grid->addWidget(new QLabel("xmax"), row, 3);
  grid->addWidget(exMax = new DoubleEdit(td->xMax(), format, prec), row, 4);
  exMax->setMinimumWidth(w);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row,
                                5);
  grid->addWidget(new QLabel(QString::fromUtf8("\316\224x")), row, 6);
  grid->addWidget(exStep = new DoubleEdit(td->deltaX(), format, prec), row, 7);
  exStep->setMinimumWidth(w);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row,
                                8);
  grid->addWidget(logX = new QCheckBox("log"), row++, 9);
  logX->setChecked(td->logX());
  grid->addWidget(new QLabel("ymin"), row, 0);
  grid->addWidget(eyMin = new DoubleEdit(td->yMin()), row, 1);
  grid->addWidget(new QLabel("ymax"), row, 3);
  grid->addWidget(eyMax = new DoubleEdit(td->yMax()), row, 4);
  grid->addWidget(logY = new QCheckBox("log"), row++, 9);
  logY->setChecked(td->logY());
  grid->addWidget(new QLabel("zmin"), row, 0);
  grid->addWidget(ezMin = new DoubleEdit(td->zMin(), format, prec), row, 1);
  ezMin->setMinimumWidth(w);
  grid->addWidget(new QLabel("zmax"), row, 3);
  grid->addWidget(ezMax = new DoubleEdit(td->zMax(), format, prec), row, 4);
  ezMax->setMinimumWidth(w);
  grid->addWidget(new QLabel(QString::fromUtf8("\316\224z")), row, 6);
  grid->addWidget(ezStep = new DoubleEdit(td->deltaZ(), format, prec), row, 7);
  ezStep->setMinimumWidth(w);
  grid->addWidget(logZ = new QCheckBox("log"), row, 9);
  logZ->setChecked(td->logZ());
  connect(bg1, SIGNAL(buttonClicked(int)), SLOT(slotEdited()));
  connect(bg2, SIGNAL(buttonClicked(int)), SLOT(slotEdited()));
  connect(logX, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(logY, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(logZ, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(bAuto, SIGNAL(clicked()), SLOT(slotAutoScale()));
  connect(ex0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(exl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(ey0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eyl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(exMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exStep, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezStep, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(efx, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(efy, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(efz, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ePhi, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eTheta, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eXref, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eYref, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eBoxx, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eBoxy, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eBoxz, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(emTicx, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(emTicy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(emTicz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigx, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ex, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ey, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ez, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(eRelSiz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(colFrame, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(colGrid, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(colData, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(modeFrame, SIGNAL(toggled(bool)), SLOT(slotEdited()));
  connect(eLetX, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetY, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetZ, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetH, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
}

void ThreeDDlg::keyPressEvent(QKeyEvent* e)
{
  if ((e->QInputEvent::modifiers() == Qt::NoModifier) &&
      (e->key() == Qt::Key_F1)) {
    buttonBox()->button(QDialogButtonBox::Help)->animateClick();
    e->accept();
  } else
    QDialog::keyPressEvent(e);
}

void ThreeDDlg::slotEdited()
{
  buttonBox()->button(QDialogButtonBox::Apply)->setEnabled(true);
  buttonBox()->button(QDialogButtonBox::Ok)->setEnabled(true);
}

void ThreeDDlg::slotEditLetX()
{
  EditTextDlg::editText(this, eLetX);
}

void ThreeDDlg::slotEditLetY()
{
  EditTextDlg::editText(this, eLetY);
}

void ThreeDDlg::slotEditLetZ()
{
  EditTextDlg::editText(this, eLetZ);
}

void ThreeDDlg::slotEditLetH()
{
  EditTextDlg::editText(this, eLetH);
}

void ThreeDDlg::slotLogToggled(bool state)
{
  const QCheckBox* b = (const QCheckBox*) sender();
  if ((b == logX) || (b == logY) || (b == logZ)) {
    SpinBox* nd;
    DoubleEdit *mi, *ma;
    if (b == logX) {
      nd = enDigx;
      mi = exMin;
      ma = exMax;
    } else
      if (b == logY) {
        nd = enDigy;
        mi = eyMin;
        ma = eyMax;
      } else {
        nd = enDigz;
        mi = ezMin;
        ma = ezMax;
      }
    nd->setEnabled(!state);
    double bottom = state ? 1e-300 : -1e300;
    mi->setBottom(bottom);
    ma->setBottom(bottom);
  }
  if (state)
    autoNorm->setChecked(false);
  slotEdited();
}

void ThreeDDlg::slotNormToggled(bool state)
{
  if (state) {
    logX->setChecked(false);
    logY->setChecked(false);
    logZ->setChecked(false);
  }
}

Fun3DDlg::Fun3DDlg(QWidget* parent, KplDoc* model, Fun3DItem *fi0,
                   const QPoint& pos) :
 ThreeDDlg(parent, model, fi0, pos, i18n("3D function"), i18n("Function")),
           fi(fi0), fit(new Fun3DItem(*fi0))
{
  QGroupBox* g = new QGroupBox("y(x,z)");
  QGridLayout* grid = new QGridLayout;
  g->setLayout(grid);
  vbox1->addWidget(g);
  grid->addWidget(new QLabel(i18n("Library")), 0, 0);
  grid->addWidget(fileName = new KUrlRequester(fi->urlY()), 0, 1);
  fileName->setStartDir(Utils::url(KplSettings::currentDir()));
  fileName->setFilter("*.so\n*");
  fileName->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
  grid->addWidget(new QLabel(i18n("Function")), 1, 0);
  QHBoxLayout* hbox = new QHBoxLayout;
  grid->addLayout(hbox, 1, 1);
  hbox->addWidget(func = new QLineEdit(fi->nameY()));
  QPushButton* tb = new QPushButton("...");
  tb->setFixedSize(fileName->button()->size());
  hbox->addWidget(tb);
  connect(tb, SIGNAL(clicked()), SLOT(slotFunc()));
  grid->addWidget(new QLabel(i18n("Parameter")), 2, 0);
  grid->addLayout(hbox = new QHBoxLayout, 2, 1);
  QPushButton* b;
  hbox->addWidget(b = new QPushButton(i18n("Edit")));
  connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Load")));
  connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Save")));
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
  initRange(new QGridLayout);
  vbox1->addWidget(g1);
  connect(fileName, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(func, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(buttonBox(), SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
  connect(bExport, SIGNAL(clicked()), SLOT(slotExport()));
  QTimer::singleShot(0, this, SLOT(slotRestoreSize()));
}

Fun3DDlg::~Fun3DDlg()
{
  KConfigGroup config = KSharedConfig::openConfig()->group("Func3DDialog");
  KWindowConfig::saveWindowSize(windowHandle(), config);
  delete fit;
}

void Fun3DDlg::apply(bool ok)
{
  if (fit->setFuncY(fileName->url(), func->text())) {
    Fun3DItem* fiNew = new Fun3DItem(*fit);
    getValues(fiNew);
    m->undoStack()->push(new EditCommand(fiNew, fi, m));
    fi = fiNew;
  }
  if (ok)
    accept();
  else {
    buttonBox()->button(QDialogButtonBox::Apply)->setEnabled(false);
    buttonBox()->button(QDialogButtonBox::Ok)->setEnabled(false);
  }
}

bool Fun3DDlg::getTemp()
{
  if (fit->setFuncY(fileName->url(), func->text())) {
    setTemp();
    return true;
  } else
    return false;
}

void Fun3DDlg::setTemp()
{
  fit->setXMin(exMin->value());
  fit->setXMax(exMax->value());
  fit->setZMin(ezMin->value());
  fit->setZMax(ezMax->value());
}

void Fun3DDlg::slotApply()
{
  apply();
}

void Fun3DDlg::slotAutoScale()
{
  if (getTemp()) {
    autoScaleXZ(fit->xMin(), fit->xMax(), fit->zMin(), fit->zMax());
    setTemp();
    int nv = fit->calcTable();
    double ymi, yma;
    ScaledItem::limits(ymi, yma, fit->yValues()[0], nv);
    autoScaleY(ymi, yma);
    slotEdited();
  }
}

void Fun3DDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox()->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
    case QDialogButtonBox::Ok:
      apply(id == QDialogButtonBox::Ok);
      break;
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/func3d.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}

void Fun3DDlg::slotEditParameter()
{
  EditParameterDlg dlg(this, fit->parY(), fileName->url().path(), func->text(),
                       true);
  connect(&dlg, SIGNAL(applyClicked()), SLOT(slotApply()));
  if (dlg.exec())
    slotEdited();
}

void Fun3DDlg::slotExport()
{
  QUrl url;
  if (KplUtils::getWriteUrl(this, url, KplSettings::currentDir())) {
    QFile f(url.isLocalFile() ? url.path() : Utils::tempFileName());
    bool success = f.open(QIODevice::WriteOnly);
    if (success) {
      QTextStream ts(&f);
      fit->exportTable(ts, KplDoc::separator(), KplDoc::presentation(),
                       KplSettings::precision());
      f.close();
      if (!url.isLocalFile())
        success = Utils::move(f.fileName(), url, this);
      if (success)
        KplUtils::setCurrentDir(url);
    }
    if (!success)
      KMessageBox::error(this, i18n("while trying to open"));
  }
}

void Fun3DDlg::slotFunc()
{
  KplUtils::getFunc(this, fileName->url(), func);
}

void Fun3DDlg::slotGetParameter()
{
  m->getPar(this, fit->parY());
}

void Fun3DDlg::slotRestoreSize()
{
  KWindowConfig::restoreWindowSize(windowHandle(),
    KSharedConfig::openConfig()->group("Func3DDialog"));
}

void Fun3DDlg::slotSaveParameter()
{
  m->saveFunPar(this, fit->parY());
}

Array3DDlg::Array3DDlg(QWidget* parent, KplDoc* model, Array3DItem *ai0,
                       const QPoint& pos) :
 ThreeDDlg(parent, model, ai0, pos, i18n("3D array"), i18n("Array")), ai(ai0),
           ait(new Array3DItem(*ai0))
{
  QGroupBox* g = new QGroupBox(i18n("Data"));
  QGridLayout* grid = new QGridLayout;
  g->setLayout(grid);
  vbox1->addWidget(g);
  grid->addWidget(new QLabel(i18n("File")), 0, 0);
  QHBoxLayout* hbox;
  grid->addLayout(hbox = new QHBoxLayout, 0, 1, 1, 3);
  hbox->addWidget(fileName = new QLabel);
  bool b_int = ai->internal();
  if (!b_int) {
    const QUrl& url = ai->url();
    fileName->setText(url.isLocalFile() ? url.path() : url.url());
  }
  fileName->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  fileName->setSizePolicy(QSizePolicy::MinimumExpanding,
                          QSizePolicy::Minimum);
  QToolButton* tb = new QToolButton;
  tb->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
  hbox->addWidget(tb);
  connect(tb, SIGNAL(clicked()), SLOT(slotFile()));
  hbox->addWidget(bReload = new QToolButton);
  bReload->setIcon(QIcon::fromTheme(QStringLiteral("view-refresh")));
  bReload->setDisabled(fileName->text().isEmpty());
  connect(bReload, SIGNAL(clicked()), SLOT(slotReload()));
  grid->addLayout(hbox = new QHBoxLayout, 1, 1);
  hbox->addWidget(internal = new QCheckBox(i18n("Internal data")));
  internal->setChecked(ai->internal());
  hbox->addWidget(edit = new QPushButton(i18n("&Edit")));
  edit->setEnabled(ai->internal());
  connect(edit, SIGNAL(clicked()), SLOT(slotEdit()));
  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding));
  vbox1->addWidget(g = new QGroupBox(i18n("Columns")));
  grid = new QGridLayout(g);
  grid->addWidget(new QLabel(i18n("x column")), 0, 0);
  grid->addWidget(ix = new QSpinBox, 0, 1);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid->addWidget(new QLabel(i18n("y column")), 0, 3);
  grid->addWidget(iy = new QSpinBox, 0, 4);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 5);
  grid->addWidget(new QLabel(i18n("z column")), 0, 6);
  grid->addWidget(iz = new QSpinBox, 0, 7);
  grid->addWidget(new QLabel(i18n("Error column")), 1, 0);
  grid->addWidget(ie = new QSpinBox, 1, 1);
  ie->setEnabled(ai->errorBars());
  grid->addWidget(err = new QCheckBox(i18n("Error bar")), 1, 2, 1, 4);
  err->setChecked(ai->errorBars());
  grid->addWidget(new QLabel(i18n("Smoothing factor")), 2, 0);
  grid->addWidget(eFactor = new DoubleEdit(ai->smf()), 2, 1);
  vbox1->addWidget(g1);
  grid = new QGridLayout(g1);
  grid->addWidget(new QLabel(i18n("Start index")), 0, 0);
  grid->addWidget(iStart = new QSpinBox, 0, 1);
  grid->addWidget(new QLabel(i18n("Number of points")), 0, 3);
  grid->addWidget(n = new QSpinBox, 0, 4);
  setSpinBoxes(ai);
  initRange(grid, 1);
  connect(ix, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(iy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(iz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ie, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(internal, SIGNAL(clicked(bool)), SLOT(slotInternal(bool)));
  connect(err, SIGNAL(toggled(bool)), ie, SLOT(setEnabled(bool)));
  connect(err, SIGNAL(clicked(bool)), SLOT(slotEdited()));
  connect(eFactor, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(iStart, SIGNAL(valueChanged(int)), SLOT(slotStartChanged(int)));
  connect(n, SIGNAL(valueChanged(int)), SLOT(slotNChanged(int)));
  connect(buttonBox(), SIGNAL(clicked(QAbstractButton*)),
          SLOT(slotButtonClicked(QAbstractButton*)));
  connect(bExport, SIGNAL(clicked()), SLOT(slotExport()));
  QTimer::singleShot(0, this, SLOT(slotRestoreSize()));
}

Array3DDlg::~Array3DDlg()
{
  KConfigGroup config = KSharedConfig::openConfig()->group("Array3DDialog");
  KWindowConfig::saveWindowSize(windowHandle(), config);
  delete ait;
}

void Array3DDlg::apply(bool ok)
{
  ait->setColX(ix->value());
  ait->setColY(iy->value());
  ait->setColZ(iz->value());
  ait->setColErr(ie->value());
  ait->setSmf(eFactor->value());
  ait->setUrl(QUrl::fromUserInput(fileName->text()));
  ait->setErrorBars(err->isChecked());
  ait->setInternal(internal->isChecked());
  int istart = iStart->value();
  ait->setRowStart(istart);
  ait->setCount(qMin(n->value(), ait->numRows() - istart));
  Array3DItem* aiNew = new Array3DItem(*ait);
  getValues(aiNew);
  m->undoStack()->push(new EditCommand(aiNew, ai, m));
  ai = aiNew;
  if (ok)
    accept();
  else {
    buttonBox()->button(QDialogButtonBox::Apply)->setEnabled(false);
    buttonBox()->button(QDialogButtonBox::Ok)->setEnabled(false);
  }
}

bool Array3DDlg::load(const QUrl &url, ArrayItem::DecimalCharacter idec)
{
  bool success = !url.isEmpty();
  if (success) {
    ait->setUrl(url);
    ait->setDecimalCharacter(idec);
    success = readData();
  }
  return success;
}

bool Array3DDlg::readData(bool fromBuffer)
{
  int ncols;
  ait->setNumRows(fromBuffer ?
                  ArrayItem::readFile(&buf, ncols, ait->data()) :
                  ArrayItem::readFile(ait->url(), ncols, ait->data(),
                                      ait->decimalCharacter()));
  if (ait->numRows()) {
    ait->setNumCols(ncols);
    int maxCol = ncols - 1;
    ait->setColX(qMin(ai->colX(), maxCol));
    ait->setColY(qMin(ai->colY(), maxCol));
    ait->setColZ(qMin(ai->colZ(), maxCol));
    ait->setColErr(qMin(ai->colErr(), maxCol));
    ait->setRowStart(qMin(ai->rowStart(), ait->numRows() - 1));
    ait->setCount(ait->numRows() - ait->rowStart());
    if (ai->count() && (!fromBuffer))
      ait->setCount(qMin(ai->count(), ait->count()));
    return true;
  } else {
    KMessageBox::error(this, i18n("while reading the data"));
    return false;
  }
}

void Array3DDlg::setSpinBoxes(Array3DItem* ad)
{
  int iColMax = qMax(0, ad->numCols() - 1);
  ix->setRange(0, iColMax);
  ix->setValue(ad->colX());
  iy->setRange(0, iColMax);
  iy->setValue(ad->colY());
  iz->setRange(0, iColMax);
  iz->setValue(ad->colZ());
  ie->setRange(0, iColMax);
  ie->setValue(ad->colErr());
  iStart->setRange(0, qMax(0, ad->numRows() - 1));
  iStart->setValue(ad->rowStart());
  n->setRange(0, ad->numRows());
  n->setValue(ad->count());
}

void Array3DDlg::slotAutoScale()
{
  if (*ait->data()) {
    double xmi, xma, ymi, yma, zmi, zma;
    int istart = iStart->value();
    int nv = n->value();
    ScaledItem::limits(xmi, xma, &(*ait->data())[ix->value()][istart], nv);
    ScaledItem::limits(ymi, yma, &(*ait->data())[iy->value()][istart], nv);
    ScaledItem::limits(zmi, zma, &(*ait->data())[iz->value()][istart], nv);
    autoScaleXZ(xmi, xma, zmi, zma);
    autoScaleY(ymi, yma);
    slotEdited();
  }
}

void Array3DDlg::slotButtonClicked(QAbstractButton* button)
{
  int id = buttonBox()->standardButton(button);
  switch (id) {
    case QDialogButtonBox::Apply:
    case QDialogButtonBox::Ok:
      apply(id == QDialogButtonBox::Ok);
      break;
    case QDialogButtonBox::Help:
      QDesktopServices::openUrl(QUrl(QStringLiteral("help:/kpl/"
                                                    "array3d.html")));
      break;
    case QDialogButtonBox::Cancel:
      reject();
  }
}

void Array3DDlg::slotEdit()
{
  buf.clear();
  QTextStream t(&buf, QIODevice::WriteOnly);
  char format = KplDoc::presentation();
  int prec = KplSettings::precision();
  char sep = KplDoc::separator();
  int ncols = ait->numCols();
  for (int i = 0; i < ait->numRows(); ++i)
    for (int j = 0; j < ncols; ++j) {
      t << QString::number((*ait->data())[j][i], format, prec);
      if (j < (ncols - 1))
        t << sep;
      else
        t << "\n";
    }
  EditDlg dlg(this, &buf, prec + 4);
  connect(&dlg, SIGNAL(applyClicked()), SLOT(slotEditApply()));
  if (dlg.exec())
    if (readData(true))
      setSpinBoxes(ait);
}

void Array3DDlg::slotEditApply()
{
  if (readData(true)) {
    setSpinBoxes(ait);
    apply();
  }
}

void Array3DDlg::slotExport()
{
  QUrl url = Utils::url(fileName->text());
  Array3DItem* aisav = ait;
  ait = new Array3DItem(*aisav);
  if (load(url, ait->decimalCharacter())) {
    if (KplUtils::getWriteUrl(this, url, KplSettings::currentDir())) {
      QFile f(url.isLocalFile() ? url.path() : Utils::tempFileName());
      bool success = f.open(QIODevice::WriteOnly);
      if (success) {
        QTextStream ts(&f);
        int maxCol = ait->numCols() - 1;
        ait->setColX(qMin(ix->value(), maxCol));
        ait->setColY(qMin(iy->value(), maxCol));
        ait->setColZ(qMin(iz->value(), maxCol));
        ait->setColErr(qMin(ie->value(), maxCol));
        ait->setRowStart(qBound(0, iStart->value(), ait->numRows() - 1));
        ait->setCount(qMin(n->value(), ait->numRows() - ait->rowStart()));
        ait->setSmf(eFactor->value());
        ait->setErrorBars(err->isChecked());
        getValues(ait);
        ait->exportTable(ts, KplDoc::separator(), KplDoc::presentation(),
                         KplSettings::precision());
        f.close();
        if (!url.isLocalFile())
          success = Utils::move(f.fileName(), url, this);
        if (success)
          KplUtils::setCurrentDir(url);
      }
      if (!success)
        KMessageBox::error(this, i18n("while trying to open"));
    }
  }
  delete ait;
  ait = aisav;
}

void Array3DDlg::slotFile()
{
  ArrayItem::DecimalCharacter idec = ait->decimalCharacter();
  QUrl url = FileDialog::getReadUrl(idec, KplSettings::currentDir() +
                                    ait->url().fileName(), this);
  if (load(url, idec)) {
    setSpinBoxes(ait);
    fileName->setText(url.isLocalFile() ? url.path() : url.url());
    KplUtils::setCurrentDir(url);
    bReload->setDisabled(fileName->text().isEmpty());
    internal->setChecked(false);
    edit->setEnabled(false);
    slotEdited();
  }
}

void Array3DDlg::slotInternal(bool state)
{
  Array3DItem* src = 0;
  if (state) {
    src = new Array3DItem(*ait);
    ait->setInternal(state);
    ait->setNumCols(4);
    ait->setRowStart(iStart->value());
    ait->setCount(n->value());
    ArrayItem::freeArray(ait->data());
    int dim = ait->numCols() * ait->numRows();
    if (dim) {
      double* x1 = new double[dim];
      memset(x1, 0, dim * sizeof(double));
      *ait->data() = new double*[ait->numCols()];
      for (int j = 0; j < ait->numCols(); ++j)
        (*ait->data())[j] = &x1[j * ait->numRows()];
      if (src->numCols() > 4) {
        int siz = ait->numRows() * sizeof(double);
        memcpy((*ait->data())[0], (*src->data())[ix->value()], siz);
        memcpy((*ait->data())[1], (*src->data())[iy->value()], siz);
        memcpy((*ait->data())[2], (*src->data())[iz->value()], siz);
        memcpy((*ait->data())[3], (*src->data())[ie->value()], siz);
        ait->setColX(0);
        ait->setColY(1);
        ait->setColZ(2);
        ait->setColErr(3);
      } else {
        memcpy((*ait->data())[0], (*src->data())[0],
               src->numCols() * src->numRows() * sizeof(double));
        ait->setColX(ix->value());
        ait->setColY(iy->value());
        ait->setColZ(iz->value());
        ait->setColErr(ie->value());
      }
    }
    setSpinBoxes(ait);
    fileName->clear();
    ait->setDecimalCharacter(ArrayItem::Point);
  } else {
    QUrl url;
    if (KplUtils::getWriteUrl(this, url, KplSettings::currentDir())) {
      QFile f(url.isLocalFile() ? url.path() : Utils::tempFileName());
      bool success = f.open(QIODevice::WriteOnly);
      if (success) {
        src = new Array3DItem(*ait);
        ait->setInternal(state);
        QTextStream t(&f);
        char format = KplDoc::presentation();
        int prec = KplSettings::precision();
        for (int i = 0; i < src->numRows(); ++i)
          for (int j = 0; j < src->numCols(); ++j)
            t << QString::number((*src->data())[j][i], format, prec)
              << (j < (src->numCols() - 1) ? " " : "\n");
        f.close();
        fileName->setText(url.isLocalFile() ? url.path() : url.url());
        if (!url.isLocalFile())
          success = Utils::move(f.fileName(), url, this);
        if (success)
          KplUtils::setCurrentDir(url);
      }
      if (!success) {
        KMessageBox::error(this, i18n("while trying to open file"));
        internal->setChecked(true);
        return;
      }
    } else {
      internal->setChecked(true);
      return;
    }
  }
  delete src;
  edit->setEnabled(state);
  bReload->setDisabled(fileName->text().isEmpty());
  slotEdited();
}

void Array3DDlg::slotNChanged(int na)
{
  int imin = ai->numRows() - na;
  if (iStart->value() > imin)
    iStart->setValue(imin);
  slotEdited();
}

void Array3DDlg::slotReload()
{
  load(Utils::url(fileName->text()), ait->decimalCharacter());
  setSpinBoxes(ait);
  slotEdited();
}

void Array3DDlg::slotRestoreSize()
{
  KWindowConfig::restoreWindowSize(windowHandle(),
    KSharedConfig::openConfig()->group("Array3DDialog"));
}

void Array3DDlg::slotStartChanged(int ia)
{
  int nmax = ai->numRows() - ia;
  if (n->value() > nmax)
    n->setValue(nmax);
  slotEdited();
}
